@@ -15,6 +15,7 @@ running multiple build commands in parallel, confusion can occur).
 import argparse
 import logging
 import sys
+import time
 
 from tools import shared
 from tools import system_libs
@@ -42,10 +43,15 @@ MINIMAL_TASKS = [
     'libdlmalloc',
     'libdlmalloc-debug',
     'libemmalloc',
-    'libemmalloc-64bit',
-    'libpthread_stub',
+    'libemmalloc-debug',
+    'libemmalloc-memvalidate',
+    'libemmalloc-verbose',
+    'libemmalloc-memvalidate-verbose',
+    'libgl',
+    'libhtml5',
     'libsockets',
     'libc_rt_wasm',
+    'libc_rt_wasm-optz',
     'struct_info',
     'libstandalonewasm',
     'crt1',
@@ -58,6 +64,7 @@ USER_TASKS = [
     'bzip2',
     'cocos2d',
     'freetype',
+    'giflib',
     'harfbuzz',
     'icu',
     'libjpeg',
@@ -97,15 +104,18 @@ Issuing 'embuilder.py build ALL' causes each task to be built.
 ''' % '\n        '.join(all_tasks)
 
 
-def build_port(port_name, lib_name):
+def build_port(port_name):
   if force:
-    shared.Cache.erase_file(lib_name)
+    system_libs.clear_port(port_name, shared.Settings)
 
   system_libs.build_port(port_name, shared.Settings)
 
 
 def main():
   global force
+
+  all_build_start_time = time.time()
+
   parser = argparse.ArgumentParser(description=__doc__,
                                    formatter_class=argparse.RawDescriptionHelpFormatter,
                                    epilog=get_help())
@@ -130,20 +140,14 @@ def main():
 
   if args.lto:
     shared.Settings.LTO = "full"
-    # Reconfigure the cache dir to reflect the change
-    shared.reconfigure_cache()
 
   if args.pic:
     shared.Settings.RELOCATABLE = 1
-    # Reconfigure the cache dir to reflect the change
-    shared.reconfigure_cache()
 
   if args.force:
     force = True
 
   # process tasks
-  libname = system_libs.Ports.get_lib_name
-
   auto_tasks = False
   tasks = args.targets
   if 'SYSTEM' in tasks:
@@ -159,109 +163,115 @@ def main():
     tasks = SYSTEM_TASKS + USER_TASKS
     auto_tasks = True
   if auto_tasks:
-    skip_tasks = []
-    if shared.Settings.RELOCATABLE:
-      # we don't support PIC + pthreads yet
-      for task in SYSTEM_TASKS + USER_TASKS:
-        if '-mt' in task:
-          skip_tasks.append(task)
-        if 'pthread' in task and 'stub' not in task:
-          skip_tasks.append(task)
-      print('Skipping building of %s, because we don\'t support threads and PIC code.' % ', '.join(skip_tasks))
     # cocos2d: must be ported, errors on
     # "Cannot recognize the target platform; are you targeting an unsupported platform?"
-    skip_tasks += ['cocos2d']
+    skip_tasks = ['cocos2d']
     tasks = [x for x in tasks if x not in skip_tasks]
     print('Building targets: %s' % ' '.join(tasks))
   for what in tasks:
     logger.info('building and verifying ' + what)
+    start_time = time.time()
     if what in SYSTEM_LIBRARIES:
       library = SYSTEM_LIBRARIES[what]
       if force:
         library.erase()
       library.get_path()
+    elif what == 'sysroot':
+      if force:
+        shared.Cache.erase_file('sysroot_install.stamp')
+      system_libs.ensure_sysroot()
     elif what == 'struct_info':
       if force:
         shared.Cache.erase_file('generated_struct_info.json')
       emscripten.generate_struct_info()
     elif what == 'icu':
-      build_port('icu', libname('libicuuc'))
+      build_port('icu')
     elif what == 'zlib':
       shared.Settings.USE_ZLIB = 1
-      build_port('zlib', 'libz.a')
+      build_port('zlib')
       shared.Settings.USE_ZLIB = 0
     elif what == 'bzip2':
-      build_port('bzip2', 'libbz2.a')
+      build_port('bzip2')
     elif what == 'bullet':
-      build_port('bullet', libname('libbullet'))
+      build_port('bullet')
     elif what == 'vorbis':
-      build_port('vorbis', libname('libvorbis'))
+      build_port('vorbis')
     elif what == 'ogg':
-      build_port('ogg', libname('libogg'))
+      build_port('ogg')
+    elif what == 'giflib':
+      build_port('giflib')
     elif what == 'libjpeg':
-      build_port('libjpeg', libname('libjpeg'))
+      build_port('libjpeg')
     elif what == 'libpng':
-      build_port('libpng', libname('libpng'))
+      build_port('libpng')
     elif what == 'sdl2':
-      build_port('sdl2', libname('libSDL2'))
+      build_port('sdl2')
     elif what == 'sdl2-mt':
       shared.Settings.USE_PTHREADS = 1
-      build_port('sdl2', libname('libSDL2-mt'))
+      build_port('sdl2')
       shared.Settings.USE_PTHREADS = 0
     elif what == 'sdl2-gfx':
-      build_port('sdl2_gfx', libname('libSDL2_gfx'))
+      build_port('sdl2_gfx')
     elif what == 'sdl2-image':
-      build_port('sdl2_image', libname('libSDL2_image'))
+      build_port('sdl2_image')
     elif what == 'sdl2-image-png':
       shared.Settings.SDL2_IMAGE_FORMATS = ["png"]
-      build_port('sdl2_image', libname('libSDL2_image_png'))
+      build_port('sdl2_image')
       shared.Settings.SDL2_IMAGE_FORMATS = []
     elif what == 'sdl2-image-jpg':
       shared.Settings.SDL2_IMAGE_FORMATS = ["jpg"]
-      build_port('sdl2_image', libname('libSDL2_image_jpg'))
+      build_port('sdl2_image')
       shared.Settings.SDL2_IMAGE_FORMATS = []
     elif what == 'sdl2-net':
-      build_port('sdl2_net', libname('libSDL2_net'))
+      build_port('sdl2_net')
     elif what == 'sdl2-mixer':
       old_formats = shared.Settings.SDL2_MIXER_FORMATS
       shared.Settings.SDL2_MIXER_FORMATS = []
-      build_port('sdl2_mixer', libname('libSDL2_mixer'))
+      build_port('sdl2_mixer')
       shared.Settings.SDL2_MIXER_FORMATS = old_formats
     elif what == 'sdl2-mixer-ogg':
       old_formats = shared.Settings.SDL2_MIXER_FORMATS
       shared.Settings.SDL2_MIXER_FORMATS = ["ogg"]
-      build_port('sdl2_mixer', libname('libSDL2_mixer_ogg'))
+      build_port('sdl2_mixer')
       shared.Settings.SDL2_MIXER_FORMATS = old_formats
     elif what == 'sdl2-mixer-mp3':
       old_formats = shared.Settings.SDL2_MIXER_FORMATS
       shared.Settings.SDL2_MIXER_FORMATS = ["mp3"]
-      build_port('sdl2_mixer', libname('libSDL2_mixer_mp3'))
+      build_port('sdl2_mixer')
       shared.Settings.SDL2_MIXER_FORMATS = old_formats
     elif what == 'freetype':
-      build_port('freetype', 'libfreetype.a')
+      build_port('freetype')
     elif what == 'harfbuzz':
-      build_port('harfbuzz', 'libharfbuzz.a')
+      build_port('harfbuzz')
     elif what == 'harfbuzz-mt':
       shared.Settings.USE_PTHREADS = 1
-      build_port('harfbuzz', 'libharfbuzz-mt.a')
+      build_port('harfbuzz')
       shared.Settings.USE_PTHREADS = 0
     elif what == 'sdl2-ttf':
-      build_port('sdl2_ttf', libname('libSDL2_ttf'))
+      build_port('sdl2_ttf')
     elif what == 'cocos2d':
-      build_port('cocos2d', libname('libcocos2d'))
+      build_port('cocos2d')
     elif what == 'regal':
-      build_port('regal', libname('libregal'))
+      build_port('regal')
     elif what == 'regal-mt':
       shared.Settings.USE_PTHREADS = 1
-      build_port('regal', libname('libregal-mt'))
+      build_port('regal')
       shared.Settings.USE_PTHREADS = 0
     elif what == 'boost_headers':
-      build_port('boost_headers', libname('libboost_headers'))
+      build_port('boost_headers')
+    elif what == 'mpg123':
+      build_port('mpg123')
     else:
       logger.error('unfamiliar build target: ' + what)
       return 1
 
-    logger.info('...success')
+    time_taken = time.time() - start_time
+    logger.info('...success. Took %s(%.2fs)' % (('%02d:%02d mins ' % (time_taken // 60, time_taken % 60) if time_taken >= 60 else ''), time_taken))
+
+  if len(tasks) > 1:
+    all_build_time_taken = time.time() - all_build_start_time
+    logger.info('Built %d targets in %s(%.2fs)' % (len(tasks), ('%02d:%02d mins ' % (all_build_time_taken // 60, all_build_time_taken % 60) if all_build_time_taken >= 60 else ''), all_build_time_taken))
+
   return 0
 
 
