@@ -28,16 +28,41 @@ function mangleCSymbolName(f) {
   return f[0] == '$' ? f.substr(1) : '_' + f;
 }
 
-// Reverses C/JS name mangling: _foo -> foo, and foo -> $foo.
-function demangleCSymbolName(f) {
-  return f[0] == '_' ? f.substr(1) : '$' + f;
+// Splits out items that pass filter. Returns also the original sans the filtered
+function splitter(array, filter) {
+  var splitOut = array.filter(filter);
+  var leftIn = array.filter((x) => !filter(x));
+  return { leftIn: leftIn, splitOut: splitOut };
+}
+
+// Functions that start with '$' should not be exported to the wasm module.
+// They are intended to be exclusive to JS code only.
+function isJsOnlyIdentifier(ident) {
+  return ident[0] == '$';
+}
+
+function escapeJSONKey(x) {
+  if (/^[\d\w_]+$/.exec(x) || x[0] === '"' || x[0] === "'") return x;
+  assert(x.indexOf("'") < 0, 'cannot have internal single quotes in keys: ' + x);
+  return "'" + x + "'";
+}
+
+function stringifyWithFunctions(obj) {
+  if (typeof obj === 'function') return obj.toString();
+  if (obj === null || typeof obj !== 'object') return JSON.stringify(obj);
+  if (Array.isArray(obj)) {
+    return '[' + obj.map(stringifyWithFunctions).join(',') + ']';
+  } else {
+    return '{' + keys(obj).map((key) => escapeJSONKey(key) + ':' + stringifyWithFunctions(obj[key])).join(',') + '}';
+  }
 }
 
 // JSifier
-function JSify(data, functionsOnly) {
+function JSify(functionsOnly) {
   var mainPass = !functionsOnly;
+  var functionStubs = [];
 
-  var itemsDict = { type: [], GlobalVariableStub: [], functionStub: [], function: [], GlobalVariable: [], GlobalVariablePostSet: [] };
+  var itemsDict = { type: [], functionStub: [], function: [], globalVariablePostSet: [] };
 
   if (mainPass) {
     // Add additional necessary items for the main pass. We can now do this since types are parsed (types can be used through
@@ -56,9 +81,10 @@ function JSify(data, functionsOnly) {
     } else {
       libFuncsToInclude = DEFAULT_LIBRARY_FUNCS_TO_INCLUDE;
     }
-    libFuncsToInclude.forEach(function(ident) {
-      data.functionStubs.push({
-        ident: mangleCSymbolName(ident)
+    libFuncsToInclude.forEach((ident) => {
+      functionStubs.push({
+        identOrig: ident,
+        identMangled: mangleCSymbolName(ident)
       });
     });
   }
@@ -74,7 +100,7 @@ function JSify(data, functionsOnly) {
 
     // apply LIBRARY_DEBUG if relevant
     if (LIBRARY_DEBUG) {
-      snippet = modifyFunction(snippet, function(name, args, body) {
+      snippet = modifyFunction(snippet, (name, args, body) => {
         return 'function ' + name + '(' + args + ') {\n' +
                'var ret = (function() { if (runtimeDebug) err("[library call:' + finalName + ': " + Array.prototype.slice.call(arguments).map(prettyPrint) + "]");\n' +
                 body +
@@ -89,31 +115,32 @@ function JSify(data, functionsOnly) {
     // In LLVM, exceptions generate a set of functions of form __cxa_find_matching_catch_1(), __cxa_find_matching_catch_2(), etc.
     // where the number specifies the number of arguments. In Emscripten, route all these to a single function '__cxa_find_matching_catch'
     // that variadically processes all of these functions using JS 'arguments' object.
-    if (item.ident.startsWith('___cxa_find_matching_catch_')) {
+    if (item.identMangled.startsWith('___cxa_find_matching_catch_')) {
       if (DISABLE_EXCEPTION_THROWING) {
         error('DISABLE_EXCEPTION_THROWING was set (likely due to -fno-exceptions), which means no C++ exception throwing support code is linked in, but exception catching code appears. Either do not set DISABLE_EXCEPTION_THROWING (if you do want exception throwing) or compile all source files with -fno-except (so that no exceptions support code is required); also make sure DISABLE_EXCEPTION_CATCHING is set to the right value - if you want exceptions, it should be off, and vice versa.');
         return;
       }
-      var num = +item.ident.split('_').slice(-1)[0];
+      var num = +item.identMangled.split('_').slice(-1)[0];
       addCxaCatch(num);
       // Continue, with the code below emitting the proper JavaScript based on
       // what we just added to the library.
     }
 
-    function addFromLibrary(ident, dependent) {
+    function addFromLibrary(item, dependent) {
+      // dependencies can be JS functions, which we just run
+      if (typeof item == 'function') return item();
+
+      var ident = item.identOrig;
+      var finalName = item.identMangled;
+
       if (ident in addedLibraryItems) return '';
       addedLibraryItems[ident] = true;
 
-      // dependencies can be JS functions, which we just run
-      if (typeof ident == 'function') return ident();
-
       // don't process any special identifiers. These are looked up when processing the base name of the identifier.
       if (isJsLibraryConfigIdentifier(ident)) {
         return '';
       }
 
-      var finalName = mangleCSymbolName(ident);
-
       // if the function was implemented in compiled code, we just need to export it so we can reach it from JS
       if (finalName in IMPLEMENTED_FUNCTIONS) {
         EXPORTED_FUNCTIONS[finalName] = 1;
@@ -153,13 +180,11 @@ function JSify(data, functionsOnly) {
           // (not useful to warn/error multiple times)
           LibraryManager.library[ident + '__docs'] = '/** @type {function(...*):?} */';
         } else {
-          var realIdent = ident;
-
-          var target = "Module['" + mangleCSymbolName(realIdent) + "']";
+          var target = "Module['" + finalName + "']";
           var assertion = '';
           if (ASSERTIONS) {
             var what = 'function';
-            assertion += 'if (!' + target + ') abort("external symbol \'' + realIdent + '\' is missing. perhaps a side module was not linked in? if this function was expected to arrive from a system library, try to build the MAIN_MODULE with EMCC_FORCE_STDLIBS=1 in the environment");\n';
+            assertion += 'if (!' + target + ') abort("external symbol \'' + ident + '\' is missing. perhaps a side module was not linked in? if this function was expected to arrive from a system library, try to build the MAIN_MODULE with EMCC_FORCE_STDLIBS=1 in the environment");\n';
 
           }
           var functionBody = assertion + "return " + target + ".apply(null, arguments);";
@@ -176,7 +201,7 @@ function JSify(data, functionsOnly) {
         error('JS library directive ' + ident + '__deps=' + deps.toString() + ' is of type ' + typeof deps + ', but it should be an array!');
         return;
       }
-      deps.forEach(function(dep) {
+      deps.forEach((dep) => {
         if (typeof snippet === 'string' && !(dep in LibraryManager.library)) warn('missing library dependency ' + dep + ', make sure you are compiling with the right options (see #if in src/library*.js)');
       });
       var isFunction = false;
@@ -207,8 +232,8 @@ function JSify(data, functionsOnly) {
         }
       }
 
-      // If a JS library item specifies xxx_import: true, then explicitly mark that symbol to be imported
-      // to asm.js/wasm module.
+      // If a JS library item specifies xxx_import: true, then explicitly mark that symbol to be exported
+      // to wasm module.
       if (LibraryManager.library[ident + '__import']) {
         Functions.libraryFunctions[finalName] = 1;
       }
@@ -226,7 +251,7 @@ function JSify(data, functionsOnly) {
         }
         if (postset && !addedLibraryItems[postsetId]) {
           addedLibraryItems[postsetId] = true;
-          itemsDict.GlobalVariablePostSet.push({
+          itemsDict.globalVariablePostSet.push({
             JS: postset + ';'
           });
         }
@@ -238,19 +263,24 @@ function JSify(data, functionsOnly) {
       // In asm, dependencies implemented in C might be needed by JS library functions.
       // We don't know yet if they are implemented in C or not. To be safe, export such
       // special cases.
-      [LIBRARY_DEPS_TO_AUTOEXPORT].forEach(function(special) {
-        deps.forEach(function(dep) {
+      [LIBRARY_DEPS_TO_AUTOEXPORT].forEach((special) => {
+        deps.forEach((dep) => {
           if (dep == special && !EXPORTED_FUNCTIONS[dep]) {
             EXPORTED_FUNCTIONS[dep] = 1;
           }
         });
       });
-      if (VERBOSE) printErr('adding ' + finalName + ' and deps ' + deps + ' : ' + (snippet + '').substr(0, 40));
+      if (VERBOSE) {
+        printErr('adding ' + finalName + ' and deps ' + deps + ' : ' + (snippet + '').substr(0, 40));
+      }
       var identDependents = ident + "__deps: ['" + deps.join("','")+"']";
       function addDependency(dep) {
+        if (typeof dep !== 'function') {
+          dep = {identOrig: dep, identMangled: mangleCSymbolName(dep)};
+        }
         return addFromLibrary(dep, identDependents + ', referenced by ' + dependent);
       }
-      var depsText = (deps ? deps.map(addDependency).filter(function(x) { return x != '' }).join('\n') + '\n' : '');
+      var depsText = (deps ? deps.map(addDependency).filter((x) => x != '').join('\n') + '\n' : '');
       var contentText;
       if (isFunction) {
         // Emit the body of a JS library function.
@@ -261,7 +291,7 @@ function JSify(data, functionsOnly) {
           }
           var sync = proxyingMode === 'sync';
           assert(typeof original === 'function');
-          contentText = modifyFunction(snippet, function(name, args, body) {
+          contentText = modifyFunction(snippet, (name, args, body) => {
             return 'function ' + name + '(' + args + ') {\n' +
                    'if (ENVIRONMENT_IS_PTHREAD) return _emscripten_proxy_to_main_thread_js(' + proxiedFunctionTable.length + ', ' + (+sync) + (args ? ', ' : '') + args + ');\n' + body + '}\n';
           });
@@ -303,15 +333,14 @@ function JSify(data, functionsOnly) {
     }
 
     itemsDict.functionStub.push(item);
-    var shortident = demangleCSymbolName(item.ident);
-    item.JS = addFromLibrary(shortident, 'top-level compiled C/C++ code');
+    item.JS = addFromLibrary(item, 'top-level compiled C/C++ code');
   }
 
   // Final combiner
 
   function finalCombiner() {
-    var splitPostSets = splitter(itemsDict.GlobalVariablePostSet, function(x) { return x.ident && x.dependencies });
-    itemsDict.GlobalVariablePostSet = splitPostSets.leftIn;
+    var splitPostSets = splitter(itemsDict.globalVariablePostSet, (x) => x.ident && x.dependencies);
+    itemsDict.globalVariablePostSet = splitPostSets.leftIn;
     var orderedPostSets = splitPostSets.splitOut;
 
     var limit = orderedPostSets.length * orderedPostSets.length;
@@ -329,13 +358,13 @@ function JSify(data, functionsOnly) {
       }
     }
 
-    itemsDict.GlobalVariablePostSet = itemsDict.GlobalVariablePostSet.concat(orderedPostSets);
+    itemsDict.globalVariablePostSet = itemsDict.globalVariablePostSet.concat(orderedPostSets);
 
     //
 
     if (!mainPass) {
-      var generated = itemsDict.function.concat(itemsDict.type).concat(itemsDict.GlobalVariableStub).concat(itemsDict.GlobalVariable);
-      print(generated.map(function(item) { return item.JS; }).join('\n'));
+      var generated = itemsDict.function.concat(itemsDict.type);
+      print(generated.map((item) => item.JS).join('\n'));
       return;
     }
 
@@ -356,12 +385,10 @@ function JSify(data, functionsOnly) {
     var legalizedI64sDefault = legalizedI64s;
     legalizedI64s = false;
 
-    var globalsData = {functionStubs: []}
-    JSify(globalsData, true);
-    globalsData = null;
+    JSify(true);
 
-    var generated = itemsDict.functionStub.concat(itemsDict.GlobalVariablePostSet);
-    generated.forEach(function(item) { print(indentify(item.JS || '', 2)); });
+    var generated = itemsDict.functionStub.concat(itemsDict.globalVariablePostSet);
+    generated.forEach((item) => print(indentify(item.JS || '', 2)));
 
     legalizedI64s = legalizedI64sDefault;
 
@@ -417,7 +444,7 @@ function JSify(data, functionsOnly) {
   // Data
 
   if (mainPass) {
-    data.functionStubs.forEach(functionStubHandler);
+    functionStubs.forEach(functionStubHandler);
   }
 
   finalCombiner();
