@@ -5,12 +5,139 @@
 
 var LibraryDylink = {
 #if RELOCATABLE
+  $asmjsMangle: function(x) {
+    var unmangledSymbols = {{{ buildStringArray(WASM_SYSTEM_EXPORTS) }}};
+    return x.indexOf('dynCall_') == 0 || unmangledSymbols.indexOf(x) != -1 ? x : '_' + x;
+  },
+
+  $resolveGlobalSymbol__deps: ['$asmjsMangle'],
+  $resolveGlobalSymbol: function(symName, direct) {
+    var sym;
+#if !WASM_BIGINT
+    if (direct) {
+      // First look for the orig$ symbol which is the symbols without
+      // any legalization performed.   Here we look on the 'asm' object
+      // to avoid any JS wrapping of the symbol.
+      sym = Module['asm']['orig$' + symName];
+    }
+#endif
+    // Then look for the unmangled name itself.
+    if (!sym) {
+      sym = Module['asm'][symName];
+    }
+    // fall back to the mangled name on the module object which could include
+    // JavaScript functions and wrapped native functions.
+#if !WASM_BIGINT
+    if (!sym && direct) {
+      sym = Module['_orig$' + symName];
+    }
+#endif
+
+    if (!sym) {
+      sym = Module[asmjsMangle(symName)];
+    }
+
+    if (!sym && symName.indexOf('invoke_') == 0) {
+      sym = createInvokeFunction(symName.split('_')[1]);
+    }
+
+    return sym;
+  },
+
+  $GOT: {},
+
+  // Create globals to each imported symbol.  These are all initialized to zero
+  // and get assigned later in `updateGOT`
+  $GOTHandler__deps: ['$GOT'],
+  $GOTHandler: {
+    'get': function(obj, symName) {
+      if (!GOT[symName]) {
+        GOT[symName] = new WebAssembly.Global({value: 'i32', mutable: true});
+#if DYLINK_DEBUG
+        err("new GOT entry: " + symName);
+#endif
+      }
+      return GOT[symName]
+    }
+  },
+
+  $isInternalSym: function(symName) {
+    // TODO: find a way to mark these in the binary or avoid exporting them.
+    return [
+      '__cpp_exception',
+      '__wasm_apply_data_relocs',
+      '__dso_handle',
+      '__set_stack_limits'
+    ].indexOf(symName) != -1
+#if SPLIT_MODULE
+        // Exports synthesized by wasm-split should be prefixed with '%'
+        || symName[0] == '%'
+#endif
+    ;
+  },
+
+  $updateGOT__deps: ['$GOT', '$isInternalSym'],
+  $updateGOT: function(exports) {
+#if DYLINK_DEBUG
+    err("updateGOT: " + Object.keys(exports).length);
+#endif
+    for (var symName in exports) {
+      if (isInternalSym(symName)) {
+        continue;
+      }
+
+      var replace = false;
+      var value = exports[symName];
+#if !WASM_BIGINT
+      if (symName.indexOf('orig$') == 0) {
+        symName = symName.split('$')[1];
+        replace = true;
+      }
+#endif
+
+      if (!GOT[symName]) {
+        GOT[symName] = new WebAssembly.Global({value: 'i32', mutable: true});
+      }
+      if (replace || GOT[symName].value == 0) {
+        if (typeof value === 'function') {
+          GOT[symName].value = addFunctionWasm(value);
+#if DYLINK_DEBUG
+          err("updateGOT FUNC: " + symName + ' : ' + GOT[symName].value);
+#endif
+        } else if (typeof value === 'number') {
+          GOT[symName].value = value;
+        } else {
+          err("unhandled export type for `" + symName + "`: " + (typeof value));
+        }
+#if DYLINK_DEBUG
+        err("updateGOT: " + symName + ' : ' + GOT[symName].value);
+#endif
+      }
+#if DYLINK_DEBUG
+      else if (GOT[symName].value != value) {
+        err("updateGOT: EXISTING SYMBOL: " + symName + ' : ' + GOT[symName].value + " " + value);
+      }
+#endif
+    }
+#if DYLINK_DEBUG
+    err("done updateGOT");
+#endif
+  },
+
   // Applies relocations to exported things.
-  $relocateExports: function(exports, memoryBase, moduleLocal) {
+  $relocateExports__deps: ['$updateGOT'],
+  $relocateExports: function(exports, memoryBase) {
     var relocated = {};
 
     for (var e in exports) {
       var value = exports[e];
+#if SPLIT_MODULE
+      // Do not modify exports synthesized by wasm-split
+      if (e.startsWith('%')) {
+        relocated[e] = value
+        continue;
+      }
+#endif
       if (typeof value === 'object') {
         // a breaking change in the wasm spec, globals are now objects
         // https://github.com/WebAssembly/mutable-global/issues/1
@@ -20,11 +147,40 @@ var LibraryDylink = {
         value += memoryBase;
       }
       relocated[e] = value;
-      if (moduleLocal) {
-        moduleLocal['_' + e] = value;
+    }
+    updateGOT(relocated);
+    return relocated;
+  },
+
+  $reportUndefinedSymbols__deps: ['$GOT', '$resolveGlobalSymbol'],
+  $reportUndefinedSymbols: function() {
+#if DYLINK_DEBUG
+    err('reportUndefinedSymbols');
+#endif
+    for (var symName in GOT) {
+      if (GOT[symName].value == 0) {
+        var value = resolveGlobalSymbol(symName, true)
+#if ASSERTIONS
+        assert(value, 'undefined symbol `' + symName + '`. perhaps a side module was not linked in? if this global was expected to arrive from a system library, try to build the MAIN_MODULE with EMCC_FORCE_STDLIBS=1 in the environment');
+#endif
+#if DYLINK_DEBUG
+        err('assigning dynamic symbol from main module: ' + symName + ' -> ' + value);
+#endif
+        if (typeof value === 'function') {
+          GOT[symName].value = addFunctionWasm(value, value.sig);
+#if DYLINK_DEBUG
+          err('assigning table entry for : ' + symName + ' -> ' + GOT[symName].value);
+#endif
+        } else if (typeof value === 'number') {
+          GOT[symName].value = value;
+        } else {
+          assert(false, 'bad export type for `' + symName + '`: ' + (typeof value));
+        }
       }
     }
-    return relocated;
+#if DYLINK_DEBUG
+    err('done reportUndefinedSymbols');
+#endif
   },
 #endif
 
@@ -61,7 +217,7 @@ var LibraryDylink = {
     loadedLibNames: {},
   },
 
-  // Dynmamic version of shared.py:make_invoke.  This is needed for invokes
+  // Dynamic version of shared.py:make_invoke.  This is needed for invokes
   // that originate from side modules since these are not known at JS
   // generation time.
   $createInvokeFunction: function(sig) {
@@ -83,9 +239,13 @@ var LibraryDylink = {
   // because the main program needs those linked in before it runs (so we can't
   // use normally malloc from the main program to do these allocations).
 
-  // Allocate memory no even if malloc isn't ready yet.
+  // Allocate memory even if malloc isn't ready yet.
+  $getMemory__deps: ['$GOT'],
   $getMemory: function(size) {
     // After the runtime is initialized, we must only use sbrk() normally.
+#if DYLINK_DEBUG
+    err("getMemory: " + size + " runtimeInitialized=" + runtimeInitialized);
+#endif
     if (runtimeInitialized)
       return _malloc(size);
     var ret = Module['___heap_base'];
@@ -94,10 +254,11 @@ var LibraryDylink = {
     assert(end <= HEAP8.length, 'failure to getMemory - memory growth etc. is not supported there, call malloc/sbrk directly or increase INITIAL_MEMORY');
 #endif
     Module['___heap_base'] = end;
+    GOT['__heap_base'].value = end;
     return ret;
   },
 
-  // fetchBinary fetches binaray data @ url. (async)
+  // fetchBinary fetches binary data @ url. (async)
   $fetchBinary: function(url) {
     return fetch(url, { credentials: 'same-origin' }).then(function(response) {
       if (!response['ok']) {
@@ -109,19 +270,10 @@ var LibraryDylink = {
     });
   },
 
-  $asmjsMangle: function(x) {
-    var unmangledSymbols = {{{ buildStringArray(WASM_SYSTEM_EXPORTS) }}};
-    return x.indexOf('dynCall_') == 0 || unmangledSymbols.indexOf(x) != -1 ? x : '_' + x;
-  },
-
-  // Loads a side module from binary data
-  $loadWebAssemblyModule__deps: ['$loadDynamicLibrary', '$createInvokeFunction', '$getMemory', '$relocateExports', '$asmjsMangle'],
-  $loadWebAssemblyModule: function(binary, flags) {
-    var int32View = new Uint32Array(new Uint8Array(binary.subarray(0, 24)).buffer);
-    assert(int32View[0] == 0x6d736100, 'need to see wasm magic number'); // \0asm
-    // we should see the dylink section right after the magic number and wasm version
-    assert(binary[8] === 0, 'need the dylink section to be first')
-    var next = 9;
+  // returns the side module metadata as an object
+  // { memorySize, memoryAlign, tableSize, tableAlign, neededDynlibs}
+  $getDylinkMetadata: function(binary) {
+    var next = 0;
     function getLEB() {
       var ret = 0;
       var mul = 1;
@@ -133,95 +285,144 @@ var LibraryDylink = {
       }
       return ret;
     }
-    var sectionSize = getLEB();
-    assert(binary[next] === 6);                 next++; // size of "dylink" string
-    assert(binary[next] === 'd'.charCodeAt(0)); next++;
-    assert(binary[next] === 'y'.charCodeAt(0)); next++;
-    assert(binary[next] === 'l'.charCodeAt(0)); next++;
-    assert(binary[next] === 'i'.charCodeAt(0)); next++;
-    assert(binary[next] === 'n'.charCodeAt(0)); next++;
-    assert(binary[next] === 'k'.charCodeAt(0)); next++;
-    var memorySize = getLEB();
-    var memoryAlign = getLEB();
-    var tableSize = getLEB();
-    var tableAlign = getLEB();
 
+    if (binary instanceof WebAssembly.Module) {
+      var dylinkSection = WebAssembly.Module.customSections(binary, "dylink");
+      assert(dylinkSection.length != 0, 'need dylink section');
+      binary = new Int8Array(dylinkSection[0]);
+    } else {
+      var int32View = new Uint32Array(new Uint8Array(binary.subarray(0, 24)).buffer);
+      assert(int32View[0] == 0x6d736100, 'need to see wasm magic number'); // \0asm
+      // we should see the dylink section right after the magic number and wasm version
+      assert(binary[8] === 0, 'need the dylink section to be first')
+      next = 9;
+      getLEB(); //section size
+      assert(binary[next] === 6);                 next++; // size of "dylink" string
+      assert(binary[next] === 'd'.charCodeAt(0)); next++;
+      assert(binary[next] === 'y'.charCodeAt(0)); next++;
+      assert(binary[next] === 'l'.charCodeAt(0)); next++;
+      assert(binary[next] === 'i'.charCodeAt(0)); next++;
+      assert(binary[next] === 'n'.charCodeAt(0)); next++;
+      assert(binary[next] === 'k'.charCodeAt(0)); next++;
+    }
+
+    var customSection = {};
+    customSection.memorySize = getLEB();
+    customSection.memoryAlign = getLEB();
+    customSection.tableSize = getLEB();
+    customSection.tableAlign = getLEB();
+#if ASSERTIONS
+    var tableAlign = Math.pow(2, customSection.tableAlign);
+    assert(tableAlign === 1, 'invalid tableAlign ' + tableAlign);
+#endif
     // shared libraries this module needs. We need to load them first, so that
     // current module could resolve its imports. (see tools/shared.py
     // WebAssembly.make_shared_library() for "dylink" section extension format)
     var neededDynlibsCount = getLEB();
-    var neededDynlibs = [];
+    customSection.neededDynlibs = [];
     for (var i = 0; i < neededDynlibsCount; ++i) {
       var nameLen = getLEB();
       var nameUTF8 = binary.subarray(next, next + nameLen);
       next += nameLen;
       var name = UTF8ArrayToString(nameUTF8, 0);
-      neededDynlibs.push(name);
+      customSection.neededDynlibs.push(name);
     }
+    return customSection;
+  },
+
+  // Module.symbols <- libModule.symbols (flags.global handler)
+  $mergeLibSymbols__deps: ['$asmjsMangle'],
+  $mergeLibSymbols: function(libModule, libName) {
+    // add symbols into global namespace TODO: weak linking etc.
+    for (var sym in libModule) {
+      if (!libModule.hasOwnProperty(sym)) {
+        continue;
+      }
+
+      // When RTLD_GLOBAL is enable, the symbols defined by this shared object will be made
+      // available for symbol resolution of subsequently loaded shared objects.
+      //
+      // We should copy the symbols (which include methods and variables) from SIDE_MODULE to MAIN_MODULE.
+
+      var module_sym = asmjsMangle(sym);
+
+      if (!Module.hasOwnProperty(module_sym)) {
+        Module[module_sym] = libModule[sym];
+      }
+#if ASSERTIONS == 2
+      else {
+        var curr = Module[sym], next = libModule[sym];
+        // don't warn on functions - might be odr, linkonce_odr, etc.
+        if (!(typeof curr === 'function' && typeof next === 'function')) {
+          err("warning: symbol '" + sym + "' from '" + libName + "' already exists (duplicate symbol? or weak linking, which isn't supported yet?)"); // + [curr, ' vs ', next]);
+        }
+      }
+#endif
+    }
+  },
+
+  // Loads a side module from binary data or compiled Module. Returns the module's exports or a
+  // promise that resolves to its exports if the loadAsync flag is set.
+  $loadWebAssemblyModule__deps: ['$loadDynamicLibrary', '$createInvokeFunction', '$getMemory', '$relocateExports', '$resolveGlobalSymbol', '$GOTHandler', '$getDylinkMetadata'],
+  $loadWebAssemblyModule: function(binary, flags) {
+    var metadata = getDylinkMetadata(binary);
+#if ASSERTIONS
+    var originalTable = wasmTable;
+#endif
 
     // loadModule loads the wasm module after all its dependencies have been loaded.
     // can be called both sync/async.
     function loadModule() {
       // alignments are powers of 2
-      memoryAlign = Math.pow(2, memoryAlign);
-      tableAlign = Math.pow(2, tableAlign);
+      var memAlign = Math.pow(2, metadata.memoryAlign);
       // finalize alignments and verify them
-      memoryAlign = Math.max(memoryAlign, STACK_ALIGN); // we at least need stack alignment
-#if ASSERTIONS
-      assert(tableAlign === 1, 'invalid tableAlign ' + tableAlign);
-#endif
+      memAlign = Math.max(memAlign, STACK_ALIGN); // we at least need stack alignment
       // prepare memory
-      var memoryBase = alignMemory(getMemory(memorySize + memoryAlign), memoryAlign); // TODO: add to cleanups
+      var memoryBase = alignMemory(getMemory(metadata.memorySize + memAlign), memAlign); // TODO: add to cleanups
+#if DYLINK_DEBUG
+      err("loadModule: memoryBase=" + memoryBase);
+#endif
       // prepare env imports
       var env = asmLibraryArg;
       // TODO: use only __memory_base and __table_base, need to update asm.js backend
-      var table = wasmTable;
-      var tableBase = table.length;
-      var originalTable = table;
-      table.grow(tableSize);
-      assert(table === originalTable);
+      var tableBase = wasmTable.length;
+      wasmTable.grow(metadata.tableSize);
       // zero-initialize memory and table
       // The static area consists of explicitly initialized data, followed by zero-initialized data.
       // The latter may need zeroing out if the MAIN_MODULE has already used this memory area before
       // dlopen'ing the SIDE_MODULE.  Since we don't know the size of the explicitly initialized data
       // here, we just zero the whole thing, which is suboptimal, but should at least resolve bugs
       // from uninitialized memory.
-      for (var i = memoryBase; i < memoryBase + memorySize; i++) {
-        HEAP8[i] = 0;
-      }
-      for (var i = tableBase; i < tableBase + tableSize; i++) {
-        table.set(i, null);
-      }
-
-      // We resolve symbols against the global Module but failing that also
-      // against the local symbols exported a side module.  This is because
-      // a) Module sometime need to import their own symbols
-      // b) Symbols from loaded modules are not always added to the global Module.
-      var moduleLocal = {};
-
-      var resolveSymbol = function(sym, type, legalized) {
-#if WASM_BIGINT
-        assert(!legalized);
-#else
-        if (legalized) {
-          sym = 'orig$' + sym;
-        }
+#if USE_PTHREADS
+      // in a pthread the module heap was already allocated and initialized in the main thread.
+      if (!ENVIRONMENT_IS_PTHREAD) {
 #endif
+        for (var i = memoryBase; i < memoryBase + metadata.memorySize; i++) {
+          HEAP8[i] = 0;
+        }
+#if USE_PTHREADS
+      }
+#endif
+      for (var i = tableBase; i < tableBase + metadata.tableSize; i++) {
+        wasmTable.set(i, null);
+      }
 
-        var resolved = Module["asm"][sym];
+      // This is the export map that we ultimately return.  We declare it here
+      // so it can be used within resolveSymbol.  We resolve symbols against
+      // this local symbol map in the case there they are not present on the
+      // global Module object.  We need this fallback because:
+      // a) Modules sometime need to import their own symbols
+      // b) Symbols from side modules are not always added to the global namespace.
+      var moduleExports;
+
+      function resolveSymbol(sym) {
+        var resolved = resolveGlobalSymbol(sym, false);
         if (!resolved) {
-          var mangled = asmjsMangle(sym);
-          resolved = Module[mangled];
-          if (!resolved) {
-            resolved = moduleLocal[mangled];
-          }
-          if (!resolved && sym.startsWith('invoke_')) {
-            resolved = createInvokeFunction(sym.split('_')[1]);
-          }
-#if ASSERTIONS
-          assert(resolved, 'missing linked ' + type + ' `' + sym + '`. perhaps a side module was not linked in? if this global was expected to arrive from a system library, try to build the MAIN_MODULE with EMCC_FORCE_STDLIBS=1 in the environment');
-#endif
+          resolved = moduleExports[sym];
         }
+#if ASSERTIONS
+        assert(resolved, 'undefined symbol `' + sym + '`. perhaps a side module was not linked in? if this global was expected to arrive from a system library, try to build the MAIN_MODULE with EMCC_FORCE_STDLIBS=1 in the environment');
+#endif
         return resolved;
       }
 
@@ -252,108 +453,106 @@ var LibraryDylink = {
             case '__table_base':
               return tableBase;
           }
-
           if (prop in obj) {
             return obj[prop]; // already present
           }
-          if (prop.startsWith('g$')) {
-            // a global. the g$ function returns the global address.
-            var name = prop.substr(2); // without g$ prefix
-            return obj[prop] = function() {
-              return resolveSymbol(name, 'global');
-            };
-          }
-          if (prop.startsWith('fp$')) {
-            // the fp$ function returns the address (table index) of the function
-            var parts = prop.split('$');
-            assert(parts.length == 3)
-            var name = parts[1];
-            var sig = parts[2];
-#if WASM_BIGINT
-            var legalized = false;
-#else
-            var legalized = sig.indexOf('j') >= 0; // check for i64s
-#endif
-            var fp = 0;
-            return obj[prop] = function() {
-              if (!fp) {
-                var f = resolveSymbol(name, 'function', legalized);
-                fp = addFunction(f, sig);
-              }
-              return fp;
-            };
-          }
           // otherwise this is regular function import - call it indirectly
+          var resolved;
           return obj[prop] = function() {
-            return resolveSymbol(prop, 'function').apply(null, arguments);
+            if (!resolved) resolved = resolveSymbol(prop, true);
+            return resolved.apply(null, arguments);
           };
         }
       };
       var proxy = new Proxy(env, proxyHandler);
       var info = {
-        global: {
-          'NaN': NaN,
-          'Infinity': Infinity,
-        },
-        'global.Math': Math,
-        env: proxy,
+        'GOT.mem': new Proxy(asmLibraryArg, GOTHandler),
+        'GOT.func': new Proxy(asmLibraryArg, GOTHandler),
+        'env': proxy,
         {{{ WASI_MODULE_NAME }}}: proxy,
       };
-#if ASSERTIONS
-      var oldTable = [];
-      for (var i = 0; i < tableBase; i++) {
-        oldTable.push(table.get(i));
-      }
-#endif
 
-      function postInstantiation(instance, moduleLocal) {
+      function postInstantiation(instance) {
 #if ASSERTIONS
         // the table should be unchanged
-        assert(table === originalTable);
-        assert(table === wasmTable);
-        // the old part of the table should be unchanged
-        for (var i = 0; i < tableBase; i++) {
-          assert(table.get(i) === oldTable[i], 'old table entries must remain the same');
-        }
-        // verify that the new table region was filled in
-        for (var i = 0; i < tableSize; i++) {
-          assert(table.get(tableBase + i) !== undefined, 'table entry was not filled in');
-        }
+        assert(wasmTable === originalTable);
 #endif
-        var exports = relocateExports(instance.exports, memoryBase, moduleLocal);
-        // initialize the module
-        var init = exports['__post_instantiate'];
-        if (init) {
-          if (runtimeInitialized) {
-            init();
-          } else {
-            // we aren't ready to run compiled code yet
-            __ATINIT__.push(init);
+        // add new entries to functionsInTableMap
+        for (var i = 0; i < metadata.tableSize; i++) {
+          var item = wasmTable.get(tableBase + i);
+#if ASSERTIONS
+          // verify that the new table region was filled in
+          assert(item !== undefined, 'table entry was not filled in');
+#endif
+          // Ignore null values.
+          if (item) {
+            functionsInTableMap.set(item, tableBase + i);
           }
         }
-        return exports;
+        moduleExports = relocateExports(instance.exports, memoryBase);
+        if (!flags.allowUndefined) {
+          reportUndefinedSymbols();
+        }
+#if STACK_OVERFLOW_CHECK >= 2
+        moduleExports['__set_stack_limits']({{{ STACK_BASE }}} , {{{ STACK_MAX }}});
+#endif
+
+        // initialize the module
+#if USE_PTHREADS
+        // The main thread does a full __post_instantiate (which includes a call
+        // to emscripten_tls_init), but secondary threads should not call static
+        // constructors in general - emscripten_tls_init is the exception.
+        if (ENVIRONMENT_IS_PTHREAD) {
+          init = moduleExports['emscripten_tls_init'];
+          assert(init);
+#if DYLINK_DEBUG
+          out("adding to tlsInitFunctions: " + init);
+#endif
+          PThread.tlsInitFunctions.push(init);
+        } else {
+#endif
+          var init = moduleExports['__post_instantiate'];
+          if (init) {
+            if (runtimeInitialized) {
+              init();
+            } else {
+              // we aren't ready to run compiled code yet
+              __ATINIT__.push(init);
+            }
+          }
+#if USE_PTHREADS
+        }
+#endif
+        return moduleExports;
       }
 
       if (flags.loadAsync) {
+        if (binary instanceof WebAssembly.Module) {
+          var instance = new WebAssembly.Instance(binary, info);
+          return Promise.resolve(postInstantiation(instance));
+        }
         return WebAssembly.instantiate(binary, info).then(function(result) {
-          return postInstantiation(result.instance, moduleLocal);
+          return postInstantiation(result.instance);
         });
-      } else {
-        var instance = new WebAssembly.Instance(new WebAssembly.Module(binary), info);
-        return postInstantiation(instance, moduleLocal);
       }
+
+      var module = binary instanceof WebAssembly.Module ? binary : new WebAssembly.Module(binary);
+      var instance = new WebAssembly.Instance(module, info);
+      return postInstantiation(instance);
     }
 
     // now load needed libraries and the module itself.
     if (flags.loadAsync) {
-      return Promise.all(neededDynlibs.map(function(dynNeeded) {
-        return loadDynamicLibrary(dynNeeded, flags);
-      })).then(function() {
+      return metadata.neededDynlibs.reduce(function(chain, dynNeeded) {
+        return chain.then(function() {
+          return loadDynamicLibrary(dynNeeded, flags);
+        });
+      }, Promise.resolve()).then(function() {
         return loadModule();
       });
     }
 
-    neededDynlibs.forEach(function(dynNeeded) {
+    metadata.neededDynlibs.forEach(function(dynNeeded) {
       loadDynamicLibrary(dynNeeded, flags);
     });
     return loadModule();
@@ -379,16 +578,16 @@ var LibraryDylink = {
   // If a library was already loaded, it is not loaded a second time. However
   // flags.global and flags.nodelete are handled every time a load request is made.
   // Once a library becomes "global" or "nodelete", it cannot be removed or unloaded.
-  $loadDynamicLibrary__deps: ['$LDSO', '$loadWebAssemblyModule', '$asmjsMangle', '$fetchBinary'],
+  $loadDynamicLibrary__deps: ['$LDSO', '$loadWebAssemblyModule', '$asmjsMangle', '$fetchBinary', '$isInternalSym', '$mergeLibSymbols'],
   $loadDynamicLibrary: function(lib, flags) {
-    if (lib == '__self__' && !LDSO.loadedLibNames[lib]) {
+    if (lib == '__main__' && !LDSO.loadedLibNames[lib]) {
       LDSO.loadedLibs[-1] = {
         refcount: Infinity,   // = nodelete
-        name:     '__self__',
+        name:     '__main__',
         module:   Module['asm'],
         global:   true
       };
-      LDSO.loadedLibNames['__self__'] = -1;
+      LDSO.loadedLibNames['__main__'] = -1;
     }
 
     // when loadDynamicLibrary did not have flags, libraries were loaded globally & permanently
@@ -406,7 +605,7 @@ var LibraryDylink = {
         dso.global = true;
         if (dso.module !== 'loading') {
           // ^^^ if module is 'loading' - symbols merging will be eventually done by the loader.
-          mergeLibSymbols(dso.module)
+          mergeLibSymbols(dso.module, lib)
         }
       }
       // same for "nodelete"
@@ -434,7 +633,7 @@ var LibraryDylink = {
       if (flags.fs) {
         var libData = flags.fs.readFile(libFile, {encoding: 'binary'});
         if (!(libData instanceof Uint8Array)) {
-          libData = new Uint8Array(lib_data);
+          libData = new Uint8Array(libData);
         }
         return flags.loadAsync ? Promise.resolve(libData) : libData;
       }
@@ -446,11 +645,6 @@ var LibraryDylink = {
       return readBinary(libFile);
     }
 
-    // libModule <- libData
-    function createLibModule(libData) {
-      return loadWebAssemblyModule(libData, flags)
-    }
-
     // libModule <- lib
     function getLibModule() {
       // lookup preloaded cache first
@@ -463,47 +657,17 @@ var LibraryDylink = {
       // module not preloaded - load lib data and create new module from it
       if (flags.loadAsync) {
         return loadLibData(lib).then(function(libData) {
-          return createLibModule(libData);
+          return loadWebAssemblyModule(libData, flags);
         });
       }
 
-      return createLibModule(loadLibData(lib));
-    }
-
-    // Module.symbols <- libModule.symbols (flags.global handler)
-    function mergeLibSymbols(libModule) {
-      // add symbols into global namespace TODO: weak linking etc.
-      for (var sym in libModule) {
-        if (!libModule.hasOwnProperty(sym)) {
-          continue;
-        }
-
-        // When RTLD_GLOBAL is enable, the symbols defined by this shared object will be made
-        // available for symbol resolution of subsequently loaded shared objects.
-        //
-        // We should copy the symbols (which include methods and variables) from SIDE_MODULE to MAIN_MODULE.
-
-        var module_sym = asmjsMangle(sym);
-
-        if (!Module.hasOwnProperty(module_sym)) {
-          Module[module_sym] = libModule[sym];
-        }
-#if ASSERTIONS == 2
-        else {
-          var curr = Module[sym], next = libModule[sym];
-          // don't warn on functions - might be odr, linkonce_odr, etc.
-          if (!(typeof curr === 'function' && typeof next === 'function')) {
-            err("warning: symbol '" + sym + "' from '" + lib + "' already exists (duplicate symbol? or weak linking, which isn't supported yet?)"); // + [curr, ' vs ', next]);
-          }
-        }
-#endif
-      }
+      return loadWebAssemblyModule(loadLibData(lib), flags);
     }
 
     // module for lib is loaded - update the dso & global namespace
     function moduleLoaded(libModule) {
       if (dso.global) {
-        mergeLibSymbols(libModule);
+        mergeLibSymbols(libModule, lib);
       }
       dso.module = libModule;
     }
@@ -519,44 +683,53 @@ var LibraryDylink = {
     return handle;
   },
 
-  $preloadDylibs__deps: ['$loadDynamicLibrary'],
+  $preloadDylibs__deps: ['$loadDynamicLibrary', '$reportUndefinedSymbols'],
   $preloadDylibs: function() {
-    var libs = {{{ JSON.stringify(RUNTIME_LINKED_LIBS) }}};
-    if (Module['dynamicLibraries']) {
-      libs = libs.concat(Module['dynamicLibraries'])
-    }
-    if (!libs.length) {
+#if DYLINK_DEBUG
+    err('preloadDylibs');
+#endif
+    if (!dynamicLibraries.length) {
+#if DYLINK_DEBUG
+      err('preloadDylibs: no libraries to preload');
+#endif
+      reportUndefinedSymbols();
       return;
     }
+
     // if we can load dynamic libraries synchronously, do so, otherwise, preload
     if (!readBinary) {
       // we can't read binary data synchronously, so preload
       addRunDependency('preloadDylibs');
-      Promise.all(libs.map(function(lib) {
-        return loadDynamicLibrary(lib, {loadAsync: true, global: true, nodelete: true});
-      })).then(function() {
+      dynamicLibraries.reduce(function(chain, lib) {
+        return chain.then(function() {
+          return loadDynamicLibrary(lib, {loadAsync: true, global: true, nodelete: true, allowUndefined: true});
+        });
+      }, Promise.resolve()).then(function() {
         // we got them all, wonderful
         removeRunDependency('preloadDylibs');
+        reportUndefinedSymbols();
       });
       return;
     }
-    libs.forEach(function(lib) {
+
+    dynamicLibraries.forEach(function(lib) {
       // libraries linked to main never go away
-      loadDynamicLibrary(lib, {global: true, nodelete: true});
+      loadDynamicLibrary(lib, {global: true, nodelete: true, allowUndefined: true});
     });
+    reportUndefinedSymbols();
   },
 
-  // void* dlopen(const char* filename, int flag);
+  // void* dlopen(const char* filename, int flags);
   dlopen__deps: ['$DLFCN', '$FS', '$ENV'],
   dlopen__proxy: 'sync',
   dlopen__sig: 'iii',
-  dlopen: function(filenameAddr, flag) {
+  dlopen: function(filenameAddr, flags) {
     // void *dlopen(const char *file, int mode);
     // http://pubs.opengroup.org/onlinepubs/009695399/functions/dlopen.html
     var searchpaths = [];
     var filename;
     if (filenameAddr === 0) {
-      filename = '__self__';
+      filename = '__main__';
     } else {
       filename = UTF8ToString(filenameAddr);
 
@@ -580,16 +753,21 @@ var LibraryDylink = {
       }
     }
 
+    if (!(flags & ({{{ cDefine('RTLD_LAZY') }}} | {{{ cDefine('RTLD_NOW') }}}))) {
+      DLFCN.errorMsg = 'invalid mode for dlopen(): Either RTLD_LAZY or RTLD_NOW is required';
+      return 0;
+    }
+
     // We don't care about RTLD_NOW and RTLD_LAZY.
-    var flags = {
-      global:   Boolean(flag & 256),  // RTLD_GLOBAL
-      nodelete: Boolean(flag & 4096), // RTLD_NODELETE
+    var jsflags = {
+      global:   Boolean(flags & {{{ cDefine('RTLD_GLOBAL') }}}),
+      nodelete: Boolean(flags & {{{ cDefine('RTLD_NODELETE') }}}),
 
       fs: FS, // load libraries from provided filesystem
     }
 
     try {
-      return loadDynamicLibrary(filename, flags)
+      return loadDynamicLibrary(filename, jsflags)
     } catch (e) {
 #if ASSERTIONS
       err('Error in loading dynamic library ' + filename + ": " + e);
@@ -626,46 +804,40 @@ var LibraryDylink = {
     // void *dlsym(void *restrict handle, const char *restrict name);
     // http://pubs.opengroup.org/onlinepubs/009695399/functions/dlsym.html
     symbol = UTF8ToString(symbol);
+    var result;
 
-    var lib = LDSO.loadedLibs[handle];
-    if (!lib) {
-      DLFCN.errorMsg = 'Tried to dlsym() from an unopened handle: ' + handle;
-      return 0;
+    if (handle == {{{ cDefine('RTLD_DEFAULT') }}}) {
+      result = resolveGlobalSymbol(symbol, true);
+      if (!result) {
+        DLFCN.errorMsg = 'Tried to lookup unknown symbol "' + symbol + '" in dynamic lib: RTLD_DEFAULT'
+        return 0;
+      }
+    } else {
+      var lib = LDSO.loadedLibs[handle];
+      if (!lib) {
+        DLFCN.errorMsg = 'Tried to dlsym() from an unopened handle: ' + handle;
+        return 0;
+      }
+      if (!lib.module.hasOwnProperty(symbol)) {
+        DLFCN.errorMsg = 'Tried to lookup unknown symbol "' + symbol + '" in dynamic lib: ' + lib.name;
+        return 0;
+      }
+#if !WASM_BIGINT
+      result = lib.module['orig$' + symbol];
+      if (!result)
+#endif
+      result = lib.module[symbol];
     }
 
-    if (!lib.module.hasOwnProperty(symbol)) {
-      DLFCN.errorMsg = ('Tried to lookup unknown symbol "' + symbol +
-                             '" in dynamic lib: ' + lib.name);
-      return 0;
-    }
-
-    var result = lib.module[symbol];
-    if (typeof result !== 'function') {
+    if (typeof result === 'function') {
+      // Insert the function into the wasm table.  If its a direct wasm function
+      // the second argument will not be needed.  If its a JS function we rely
+      // on the `sig` attribute being set based on the `<func>__sig` specified
+      // in library JS file.
+      return addFunctionWasm(result, result.sig);
+    } else {
       return result;
     }
-
-#if EMULATE_FUNCTION_POINTER_CASTS
-    // for wasm with emulated function pointers, the i64 ABI is used for all
-    // function calls, so we can't just call addFunction on something JS
-    // can call (which does not use that ABI), as the function pointer would
-    // not be usable from wasm. instead, the wasm has exported function pointers
-    // for everything we need, with prefix fp$, use those
-    result = lib.module['fp$' + symbol];
-    if (typeof result === 'object') {
-      // a breaking change in the wasm spec, globals are now objects
-      // https://github.com/WebAssembly/mutable-global/issues/1
-      result = result.value;
-    }
-#if ASSERTIONS
-    assert(typeof result === 'number', 'could not find function pointer for ' + symbol);
-#endif // ASSERTIONS
-    return result;
-#else // EMULATE_FUNCTION_POINTER_CASTS
-    // Insert the function into the wasm table.  Since we know the function
-    // comes directly from the loaded wasm module we can insert it directly
-    // into the table, avoiding any JS interaction.
-    return addFunctionWasm(result);
-#endif // EMULATE_FUNCTION_POINTER_CASTS
   },
 
   // char* dlerror(void);
