@@ -251,6 +251,9 @@ FS.staticInit();` +
       return FS.lookup(parent, name);
     },
     createNode: function(parent, name, mode, rdev) {
+#if ASSERTIONS
+      assert(typeof parent === 'object')
+#endif
       var node = new FS.FSNode(parent, name, mode, rdev);
 
       FS.hashAddNode(node);
@@ -1329,7 +1332,7 @@ FS.staticInit();` +
       });
       FS.mkdev('/dev/null', FS.makedev(1, 3));
       // setup /dev/tty and /dev/tty1
-      // stderr needs to print output using Module['printErr']
+      // stderr needs to print output using err() rather than out()
       // so we register a second tty just for it.
       TTY.register(FS.makedev(5, 0), TTY.default_tty_ops);
       TTY.register(FS.makedev(6, 0), TTY.default_tty1_ops);
@@ -1345,13 +1348,14 @@ FS.staticInit();` +
       FS.mkdir('/dev/shm/tmp');
     },
     createSpecialDirectories: function() {
-      // create /proc/self/fd which allows /proc/self/fd/6 => readlink gives the name of the stream for fd 6 (see test_unistd_ttyname)
+      // create /proc/self/fd which allows /proc/self/fd/6 => readlink gives the
+      // name of the stream for fd 6 (see test_unistd_ttyname)
       FS.mkdir('/proc');
-      FS.mkdir('/proc/self');
+      var proc_self = FS.mkdir('/proc/self');
       FS.mkdir('/proc/self/fd');
       FS.mount({
         mount: function() {
-          var node = FS.createNode('/proc/self', 'fd', {{{ cDefine('S_IFDIR') }}} | 511 /* 0777 */, {{{ cDefine('S_IXUGO') }}});
+          var node = FS.createNode(proc_self, 'fd', {{{ cDefine('S_IFDIR') }}} | 511 /* 0777 */, {{{ cDefine('S_IXUGO') }}});
           node.node_ops = {
             lookup: function(parent, name) {
               var fd = +name;
@@ -1755,7 +1759,7 @@ FS.staticInit();` +
         Object.defineProperties(lazyArray, {
           length: {
             get: /** @this{Object} */ function() {
-              if(!this.lengthKnown) {
+              if (!this.lengthKnown) {
                 this.cacheLength();
               }
               return this._length;
@@ -1763,7 +1767,7 @@ FS.staticInit();` +
           },
           chunkSize: {
             get: /** @this{Object} */ function() {
-              if(!this.lengthKnown) {
+              if (!this.lengthKnown) {
                 this.cacheLength();
               }
               return this._chunkSize;
