@@ -1,7 +1,8 @@
-// Copyright 2013 The Emscripten Authors.  All rights reserved.
-// Emscripten is available under two separate licenses, the MIT license and the
-// University of Illinois/NCSA Open Source License.  Both these licenses can be
-// found in the LICENSE file.
+/**
+ * @license
+ * Copyright 2013 The Emscripten Authors
+ * SPDX-License-Identifier: MIT
+ */
 
 // Implementation of libuuid creating RFC4122 version 4 random UUIDs.
 
@@ -61,8 +62,9 @@ mergeInto(LibraryManager.library, {
       }
     }
 
-    uuid[6] = (uuid[6] & 0x0F) | 0x40;
-    uuid[8] = (uuid[8] & 0x7F) | 0x80;
+    // Makes uuid compliant to RFC-4122
+    uuid[6] = (uuid[6] & 0x0F) | 0x40; // uuid version
+    uuid[8] = (uuid[8] & 0x3F) | 0x80; // uuid variant
     writeArrayToMemory(uuid, out);
   },
 
