@@ -5,8 +5,7 @@
  * found in the LICENSE file.
  */
 
-#ifndef __em_js_h__
-#define __em_js_h__
+#pragma once
 
 #ifdef __cplusplus
 #define _EM_JS_CPP_BEGIN extern "C" {
@@ -27,41 +26,41 @@
 // function bodies containing commas are seamlessly handled.
 
 // EM_JS declares the JS function with a C function prototype, which becomes a
-// function import in asm.js/wasm. It also declares an __em_js__-prefixed
-// function, which we can use to pass information to the Emscripten compiler
-// that survives going through LLVM.
+// function import in wasm. It also declares an __em_js__-prefixed string constant,
+// which we can use to pass information to the Emscripten compiler that survives
+// going through LLVM.
 // Example:
+//
 //   EM_JS(int, foo, (int x, int y), { return 2 * x + y; })
+//
 // would get translated into:
-//   int foo(int x, int y);
+//
+//   __attribute__((import_name("foo"))) int foo(int x, int y);
+//
 //   __attribute__((used, visibility("default")))
-//   const char* __em_js__foo() {
-//     return "(int x, int y)<::>{ return 2 * x + y; }";
-//   }
+//   char __em_js__foo[] = "(int x, int y)<::>{ return 2 * x + y; }";
+//
 // We pack the arguments and function body into a constant string so it's
-// readable from asm.js/wasm post-processing.
-// Later we see a function called __em_js__foo, meaning we need to create a JS
+// readable during wasm post-processing.
+// Later we see an export called __em_js__foo, meaning we need to create a JS
 // function:
 //   function foo(x, y) { return 2 * x + y; }
 // We use <::> to separate the arguments from the function body because it isn't
 // valid anywhere in a C function declaration.
 
-// Generated __em_js__-prefixed functions are read by either the JSBackend (for
-// asm.js) or by Binaryen, and the string data is extracted into the Emscripten
-// metadata dictionary under the "emJsFuncs" key. emJsFuncs itself is a
-// dictionary where the keys are function names (not prefixed with __em_js__),
-// and the values are the <::>-including description strings.
+// Generated __em_js__-prefixed symbols are read by binaryen, and the string
+// data is extracted into the Emscripten metadata dictionary under the
+// "emJsFuncs" key. emJsFuncs itself is a dictionary where the keys are function
+// names (not prefixed with __em_js__), and the values are the <::>-including
+// description strings.
 
 // emJsFuncs metadata is read in emscripten.py's create_em_js, which creates an
 // array of JS function strings to be included in the JS output.
 
-#define EM_JS(ret, name, params, ...)          \
-  _EM_JS_CPP_BEGIN                             \
-  extern ret name params;                      \
-  __attribute__((used, visibility("default"))) \
-  const char* __em_js__##name() {              \
-    return #params "<::>" #__VA_ARGS__;        \
-  }                                            \
+#define EM_JS(ret, name, params, ...)                                                              \
+  _EM_JS_CPP_BEGIN                                                                                 \
+  ret name params EM_IMPORT(name);                                                                 \
+  EMSCRIPTEN_KEEPALIVE                                                                             \
+  __attribute__((section("em_js"), aligned(1))) char __em_js__##name[] =                           \
+    #params "<::>" #__VA_ARGS__;                                                                   \
   _EM_JS_CPP_END
-
-#endif // __em_js_h__
