@@ -4,19 +4,33 @@
 #include <emscripten/html5.h>
 
 // 1. https://www.khronos.org/registry/webgl/extensions/OES_texture_float/
+#ifndef EMSCRIPTEN_GL_OES_texture_float
+#define EMSCRIPTEN_GL_OES_texture_float 1
 // <no symbols exposed>
+#endif /* EMSCRIPTEN_GL_OES_texture_float */
 
 // 2. https://www.khronos.org/registry/webgl/extensions/OES_texture_half_float/
+#ifndef EMSCRIPTEN_GL_OES_texture_half_float
+#define EMSCRIPTEN_GL_OES_texture_half_float 1
 #define GL_HALF_FLOAT_OES 0x8D61
+#endif /* EMSCRIPTEN_GL_OES_texture_half_float */
 
 // 3. https://www.khronos.org/registry/webgl/extensions/WEBGL_lose_context/
+#ifndef EMSCRIPTEN_GL_WEBGL_lose_context
+#define EMSCRIPTEN_GL_WEBGL_lose_context 1
 WEBGL_APICALL EMSCRIPTEN_RESULT GL_APIENTRY emscripten_webgl_loseContext(EMSCRIPTEN_WEBGL_CONTEXT_HANDLE contextHandle);
 WEBGL_APICALL EMSCRIPTEN_RESULT GL_APIENTRY emscripten_webgl_restoreContext(EMSCRIPTEN_WEBGL_CONTEXT_HANDLE contextHandle);
+#endif /* EMSCRIPTEN_GL_WEBGL_lose_context */
 
 // 4. https://www.khronos.org/registry/webgl/extensions/OES_standard_derivatives/
+#ifndef EMSCRIPTEN_GL_OES_standard_derivatives
+#define EMSCRIPTEN_GL_OES_standard_derivatives 1
 #define GL_FRAGMENT_SHADER_DERIVATIVE_HINT_OES 0x8B8B
+#endif /* EMSCRIPTEN_GL_OES_standard_derivatives */
 
 // 5. https://www.khronos.org/registry/webgl/extensions/OES_vertex_array_object/
+#ifndef EMSCRIPTEN_GL_OES_vertex_array_object
+#define EMSCRIPTEN_GL_OES_vertex_array_object 1
 #define GL_VERTEX_ARRAY_BINDING_OES 0x85B5
 WEBGL_APICALL void GL_APIENTRY emscripten_glBindVertexArrayOES(GLuint array);
 WEBGL_APICALL void GL_APIENTRY emscripten_glDeleteVertexArraysOES(GLsizei n, const GLuint *arrays);
@@ -26,34 +40,58 @@ WEBGL_APICALL void GL_APIENTRY glBindVertexArrayOES(GLuint array);
 WEBGL_APICALL void GL_APIENTRY glDeleteVertexArraysOES(GLsizei n, const GLuint *arrays);
 WEBGL_APICALL void GL_APIENTRY glGenVertexArraysOES(GLsizei n, GLuint *arrays);
 WEBGL_APICALL GLboolean GL_APIENTRY glIsVertexArrayOES(GLuint array);
+#endif /* EMSCRIPTEN_GL_OES_vertex_array_object */
 
 // 6. https://www.khronos.org/registry/webgl/extensions/WEBGL_debug_renderer_info/
+#ifndef EMSCRIPTEN_GL_WEBGL_debug_renderer_info
+#define EMSCRIPTEN_GL_WEBGL_debug_renderer_info 1
 #define GL_UNMASKED_VENDOR_WEBGL 0x9245
 #define GL_UNMASKED_RENDERER_WEBGL 0x9246
+#endif /* EMSCRIPTEN_GL_WEBGL_debug_renderer_info */
 
 // 7. https://www.khronos.org/registry/webgl/extensions/WEBGL_debug_shaders/
+#ifndef EMSCRIPTEN_GL_WEBGL_debug_shaders
+#define EMSCRIPTEN_GL_WEBGL_debug_shaders 1
 WEBGL_APICALL void GL_APIENTRY emscripten_webgl_getTranslatedShaderSource(GLuint shader, GLsizei bufSize, GLsizei *length, GLchar *source);
+#endif /* EMSCRIPTEN_GL_WEBGL_debug_shaders */
 
 // 8. https://www.khronos.org/registry/webgl/extensions/WEBGL_compressed_texture_s3tc/
+#ifndef EMSCRIPTEN_GL_WEBGL_compressed_texture_s3tc
+#define EMSCRIPTEN_GL_WEBGL_compressed_texture_s3tc 1
 #define GL_COMPRESSED_RGB_S3TC_DXT1_EXT 0x83F0
 #define GL_COMPRESSED_RGBA_S3TC_DXT1_EXT 0x83F1
 #define GL_COMPRESSED_RGBA_S3TC_DXT3_EXT 0x83F2
 #define GL_COMPRESSED_RGBA_S3TC_DXT5_EXT 0x83F3
+#endif /* EMSCRIPTEN_GL_WEBGL_compressed_texture_s3tc */
 
 // 9. https://www.khronos.org/registry/webgl/extensions/WEBGL_depth_texture/
+#ifndef EMSCRIPTEN_GL_WEBGL_depth_texture
+#define EMSCRIPTEN_GL_WEBGL_depth_texture 1
 #define GL_UNSIGNED_INT_24_8_WEBGL 0x84FA
+#endif /* EMSCRIPTEN_GL_WEBGL_depth_texture */
 
 // 10. https://www.khronos.org/registry/webgl/extensions/OES_element_index_uint/
+#ifndef EMSCRIPTEN_GL_OES_element_index_uint
+#define EMSCRIPTEN_GL_OES_element_index_uint 1
 // <no symbols exposed>
+#endif /* EMSCRIPTEN_GL_OES_element_index_uint */
 
 // 11. https://www.khronos.org/registry/webgl/extensions/EXT_texture_filter_anisotropic/
+#ifndef EMSCRIPTEN_GL_EXT_texture_filter_anisotropic
+#define EMSCRIPTEN_GL_EXT_texture_filter_anisotropic 1
 #define GL_TEXTURE_MAX_ANISOTROPY_EXT 0x84FE
 #define GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT 0x84FF
+#endif /* EMSCRIPTEN_GL_EXT_texture_filter_anisotropic */
 
 // 16. https://www.khronos.org/registry/webgl/extensions/EXT_frag_depth/
+#ifndef EMSCRIPTEN_GL_EXT_frag_depth
+#define EMSCRIPTEN_GL_EXT_frag_depth 1
 // <no symbols exposed>
+#endif /* EMSCRIPTEN_GL_EXT_frag_depth */
 
 // 18. https://www.khronos.org/registry/webgl/extensions/WEBGL_draw_buffers/
+#ifndef EMSCRIPTEN_GL_WEBGL_draw_buffers
+#define EMSCRIPTEN_GL_WEBGL_draw_buffers 1
 #define GL_COLOR_ATTACHMENT0_WEBGL 0x8CE0
 #define GL_COLOR_ATTACHMENT1_WEBGL 0x8CE1
 #define GL_COLOR_ATTACHMENT2_WEBGL 0x8CE2
@@ -90,8 +128,11 @@ WEBGL_APICALL void GL_APIENTRY emscripten_webgl_getTranslatedShaderSource(GLuint
 #define GL_MAX_DRAW_BUFFERS_WEBGL 0x8824
 WEBGL_APICALL void GL_APIENTRY emscripten_glDrawBuffersWEBGL(GLsizei n, const GLenum *buffers);
 WEBGL_APICALL void GL_APIENTRY glDrawBuffersWEBGL(GLsizei n, const GLenum *buffers);
+#endif /* EMSCRIPTEN_GL_WEBGL_draw_buffers */
 
 // 19. https://www.khronos.org/registry/webgl/extensions/ANGLE_instanced_arrays/
+#ifndef EMSCRIPTEN_GL_ANGLE_instanced_arrays
+#define EMSCRIPTEN_GL_ANGLE_instanced_arrays 1
 #define GL_VERTEX_ATTRIB_ARRAY_DIVISOR_ANGLE 0x88FE
 WEBGL_APICALL void GL_APIENTRY emscripten_glDrawArraysInstancedANGLE(GLenum mode, GLint first, GLsizei count, GLsizei primcount);
 WEBGL_APICALL void GL_APIENTRY emscripten_glDrawElementsInstancedANGLE(GLenum mode, GLsizei count, GLenum type, GLintptr offset, GLsizei primcount);
@@ -99,52 +140,79 @@ WEBGL_APICALL void GL_APIENTRY emscripten_glVertexAttribDivisorANGLE(GLuint inde
 WEBGL_APICALL void GL_APIENTRY glDrawArraysInstancedANGLE(GLenum mode, GLint first, GLsizei count, GLsizei primcount);
 WEBGL_APICALL void GL_APIENTRY glDrawElementsInstancedANGLE(GLenum mode, GLsizei count, GLenum type, GLintptr offset, GLsizei primcount);
 WEBGL_APICALL void GL_APIENTRY glVertexAttribDivisorANGLE(GLuint index, GLuint divisor);
+#endif /* EMSCRIPTEN_GL_ANGLE_instanced_arrays */
 
 // 20. https://www.khronos.org/registry/webgl/extensions/OES_texture_float_linear/
+#ifndef EMSCRIPTEN_GL_OES_texture_float_linear
+#define EMSCRIPTEN_GL_OES_texture_float_linear 1
 // <no symbols exposed>
+#endif /* EMSCRIPTEN_GL_OES_texture_float_linear */
 
 // 21. https://www.khronos.org/registry/webgl/extensions/OES_texture_half_float_linear/
+#ifndef EMSCRIPTEN_GL_OES_texture_half_float_linear
+#define EMSCRIPTEN_GL_OES_texture_half_float_linear 1
 // <no symbols exposed>
+#endif /* EMSCRIPTEN_GL_OES_texture_half_float_linear */
 
 // 25. https://www.khronos.org/registry/webgl/extensions/EXT_blend_minmax/
+#ifndef EMSCRIPTEN_GL_EXT_blend_minmax
+#define EMSCRIPTEN_GL_EXT_blend_minmax 1
 #define GL_MIN_EXT 0x8007
 #define GL_MAX_EXT 0x8008
+#endif /* EMSCRIPTEN_GL_EXT_blend_minmax */
 
 // 27. https://www.khronos.org/registry/webgl/extensions/EXT_shader_texture_lod/
+#ifndef EMSCRIPTEN_GL_EXT_shader_texture_lod
+#define EMSCRIPTEN_GL_EXT_shader_texture_lod 1
 // <no symbols exposed>
-
-// 12. https://www.khronos.org/registry/webgl/extensions/WEBGL_compressed_texture_atc/
-#define GL_COMPRESSED_RGB_ATC_WEBGL 0x8C92
-#define GL_COMPRESSED_RGBA_ATC_EXPLICIT_ALPHA_WEBGL 0x8C93
-#define GL_COMPRESSED_RGBA_ATC_INTERPOLATED_ALPHA_WEBGL 0x87EE
+#endif /* EMSCRIPTEN_GL_EXT_shader_texture_lod */
 
 // 13. https://www.khronos.org/registry/webgl/extensions/WEBGL_compressed_texture_pvrtc/
+#ifndef EMSCRIPTEN_GL_WEBGL_compressed_texture_pvrtc
+#define EMSCRIPTEN_GL_WEBGL_compressed_texture_pvrtc 1
 #define GL_COMPRESSED_RGB_PVRTC_4BPPV1_IMG 0x8C00
 #define GL_COMPRESSED_RGB_PVRTC_2BPPV1_IMG 0x8C01
 #define GL_COMPRESSED_RGBA_PVRTC_4BPPV1_IMG 0x8C02
 #define GL_COMPRESSED_RGBA_PVRTC_2BPPV1_IMG 0x8C03
+#endif /* EMSCRIPTEN_GL_WEBGL_compressed_texture_pvrtc */
 
 // 14. https://www.khronos.org/registry/webgl/extensions/EXT_color_buffer_half_float/
+#ifndef EMSCRIPTEN_GL_EXT_color_buffer_half_float
+#define EMSCRIPTEN_GL_EXT_color_buffer_half_float 1
 #define GL_RGBA16F_EXT 0x881A
 #define GL_RGB16F_EXT 0x881B
+#define GL_RG16F_EXT 0x822F
+#define GL_R16F_EXT 0x822D
 #define GL_FRAMEBUFFER_ATTACHMENT_COMPONENT_TYPE_EXT 0x8211
 #define GL_UNSIGNED_NORMALIZED_EXT 0x8C17
+#endif /* EMSCRIPTEN_GL_EXT_color_buffer_half_float */
 
 // 15. https://www.khronos.org/registry/webgl/extensions/WEBGL_color_buffer_float/
+#ifndef EMSCRIPTEN_GL_WEBGL_color_buffer_float
+#define EMSCRIPTEN_GL_WEBGL_color_buffer_float 1
 #define GL_RGBA32F_EXT 0x8814
 #define GL_FRAMEBUFFER_ATTACHMENT_COMPONENT_TYPE_EXT 0x8211
 #define GL_UNSIGNED_NORMALIZED_EXT 0x8C17
+#endif /* EMSCRIPTEN_GL_WEBGL_color_buffer_float */
 
 // 17. https://www.khronos.org/registry/webgl/extensions/EXT_sRGB/
+#ifndef EMSCRIPTEN_GL_EXT_sRGB
+#define EMSCRIPTEN_GL_EXT_sRGB 1
 #define GL_SRGB_EXT 0x8C40
 #define GL_SRGB_ALPHA_EXT 0x8C42
 #define GL_SRGB8_ALPHA8_EXT 0x8C43
 #define GL_FRAMEBUFFER_ATTACHMENT_COLOR_ENCODING_EXT 0x8210
+#endif /* EMSCRIPTEN_GL_EXT_sRGB */
 
 // 24. https://www.khronos.org/registry/webgl/extensions/WEBGL_compressed_texture_etc1/
+#ifndef EMSCRIPTEN_GL_WEBGL_compressed_texture_etc1
+#define EMSCRIPTEN_GL_WEBGL_compressed_texture_etc1 1
 #define GL_COMPRESSED_RGB_ETC1_WEBGL 0x8D64
+#endif /* EMSCRIPTEN_GL_WEBGL_compressed_texture_etc1 */
 
 // 26. https://www.khronos.org/registry/webgl/extensions/EXT_disjoint_timer_query/
+#ifndef EMSCRIPTEN_GL_EXT_disjoint_timer_query
+#define EMSCRIPTEN_GL_EXT_disjoint_timer_query 1
 #define GL_QUERY_COUNTER_BITS_EXT 0x8864
 #define GL_CURRENT_QUERY_EXT 0x8865
 #define GL_QUERY_RESULT_EXT 0x8866
@@ -174,9 +242,11 @@ WEBGL_APICALL void GL_APIENTRY glGetQueryObjectivEXT(GLuint id, GLenum pname, GL
 WEBGL_APICALL void GL_APIENTRY glGetQueryObjectuivEXT(GLuint id, GLenum pname, GLuint *params);
 WEBGL_APICALL void GL_APIENTRY glGetQueryObjecti64vEXT(GLuint id, GLenum pname, GLint64 *params);
 WEBGL_APICALL void GL_APIENTRY glGetQueryObjectui64vEXT(GLuint id, GLenum pname, GLuint64 *params);
-
+#endif /* EMSCRIPTEN_GL_EXT_disjoint_timer_query */
 
 // 29. https://www.khronos.org/registry/webgl/extensions/WEBGL_compressed_texture_etc/
+#ifndef EMSCRIPTEN_GL_WEBGL_compressed_texture_etc
+#define EMSCRIPTEN_GL_WEBGL_compressed_texture_etc 1
 #define GL_COMPRESSED_R11_EAC 0x9270
 #define GL_COMPRESSED_SIGNED_R11_EAC 0x9271
 #define GL_COMPRESSED_RG11_EAC 0x9272
@@ -187,8 +257,11 @@ WEBGL_APICALL void GL_APIENTRY glGetQueryObjectui64vEXT(GLuint id, GLenum pname,
 #define GL_COMPRESSED_SRGB8_PUNCHTHROUGH_ALPHA1_ETC2 0x9277
 #define GL_COMPRESSED_RGBA8_ETC2_EAC 0x9278
 #define GL_COMPRESSED_SRGB8_ALPHA8_ETC2_EAC 0x9279
+#endif /* EMSCRIPTEN_GL_WEBGL_compressed_texture_etc */
 
 // 30. https://www.khronos.org/registry/webgl/extensions/WEBGL_compressed_texture_astc/
+#ifndef EMSCRIPTEN_GL_WEBGL_compressed_texture_astc
+#define EMSCRIPTEN_GL_WEBGL_compressed_texture_astc 1
 #define GL_COMPRESSED_RGBA_ASTC_4x4_KHR 0x93B0
 #define GL_COMPRESSED_RGBA_ASTC_5x4_KHR 0x93B1
 #define GL_COMPRESSED_RGBA_ASTC_5x5_KHR 0x93B2
@@ -218,27 +291,45 @@ WEBGL_APICALL void GL_APIENTRY glGetQueryObjectui64vEXT(GLuint id, GLenum pname,
 #define GL_COMPRESSED_SRGB8_ALPHA8_ASTC_12x10_KHR 0x93DC
 #define GL_COMPRESSED_SRGB8_ALPHA8_ASTC_12x12_KHR 0x93DD
 WEBGL_APICALL void GL_APIENTRY emscripten_webgl_getSupportedAstcProfiles(GLsizei bufSize, GLsizei *length, GLchar *buf);
+#endif /* EMSCRIPTEN_GL_WEBGL_compressed_texture_astc */
 
 // 31. https://www.khronos.org/registry/webgl/extensions/EXT_color_buffer_float/
+#ifndef EMSCRIPTEN_GL_EXT_color_buffer_float
+#define EMSCRIPTEN_GL_EXT_color_buffer_float 1
 // <no symbols exposed>
+#endif /* EMSCRIPTEN_GL_EXT_color_buffer_float */
 
 // 32. https://www.khronos.org/registry/webgl/extensions/WEBGL_compressed_texture_s3tc_srgb/
+#ifndef EMSCRIPTEN_GL_WEBGL_compressed_texture_s3tc_srgb
+#define EMSCRIPTEN_GL_WEBGL_compressed_texture_s3tc_srgb 1
 #define GL_COMPRESSED_SRGB_S3TC_DXT1_EXT 0x8C4C
 #define GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT1_EXT 0x8C4D
 #define GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT3_EXT 0x8C4E
 #define GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT5_EXT 0x8C4F
+#endif /* EMSCRIPTEN_GL_WEBGL_compressed_texture_s3tc_srgb */
+
+// 37. https://www.khronos.org/registry/webgl/extensions/KHR_parallel_shader_compile/
+#ifndef EMSCRIPTEN_GL_KHR_PARALLEL_SHADER_COMPILE
+#define EMSCRIPTEN_GL_KHR_PARALLEL_SHADER_COMPILE 1
+#define GL_COMPLETION_STATUS_KHR 0x91B1
+#endif
 
 // 40. https://www.khronos.org/registry/webgl/extensions/WEBGL_multi_draw/
+#ifndef EMSCRIPTEN_GL_WEBGL_multi_draw
+#define EMSCRIPTEN_GL_WEBGL_multi_draw 1
 WEBGL_APICALL void GL_APIENTRY emscripten_glMultiDrawArraysWEBGL(GLenum mode, const GLint* firsts, const GLsizei* counts, GLsizei drawcount);
 WEBGL_APICALL void GL_APIENTRY emscripten_glMultiDrawArraysInstancedWEBGL(GLenum mode, const GLint* firsts, const GLsizei* counts, const GLsizei* instanceCounts, GLsizei drawcount);
-WEBGL_APICALL void GL_APIENTRY emscripten_glMultiDrawElementsWEBGL(GLenum mode, const GLsizei* counts, GLenum type, const GLsizei* offsets, GLsizei drawcount);
-WEBGL_APICALL void GL_APIENTRY emscripten_glMultiDrawElementsInstancedWEBGL(GLenum mode, const GLsizei* counts, GLenum type, const GLsizei* offsets, const GLsizei* instanceCounts, GLsizei drawcount);
+WEBGL_APICALL void GL_APIENTRY emscripten_glMultiDrawElementsWEBGL(GLenum mode, const GLsizei* counts, GLenum type, const GLvoid* const* offsets, GLsizei drawcount);
+WEBGL_APICALL void GL_APIENTRY emscripten_glMultiDrawElementsInstancedWEBGL(GLenum mode, const GLsizei* counts, GLenum type, const GLvoid* const* offsets, const GLsizei* instanceCounts, GLsizei drawcount);
 WEBGL_APICALL void GL_APIENTRY glMultiDrawArraysWEBGL(GLenum mode, const GLint* firsts, const GLsizei* counts, GLsizei drawcount);
 WEBGL_APICALL void GL_APIENTRY glMultiDrawArraysInstancedWEBGL(GLenum mode, const GLint* firsts, const GLsizei* counts, const GLsizei* instanceCounts, GLsizei drawcount);
-WEBGL_APICALL void GL_APIENTRY glMultiDrawElementsWEBGL(GLenum mode, const GLsizei* counts, GLenum type, const GLsizei* offsets, GLsizei drawcount);
-WEBGL_APICALL void GL_APIENTRY glMultiDrawElementsInstancedWEBGL(GLenum mode, const GLsizei* counts, GLenum type, const GLsizei* offsets, const GLsizei* instanceCounts, GLsizei drawcount);
+WEBGL_APICALL void GL_APIENTRY glMultiDrawElementsWEBGL(GLenum mode, const GLsizei* counts, GLenum type, const GLvoid* const* offsets, GLsizei drawcount);
+WEBGL_APICALL void GL_APIENTRY glMultiDrawElementsInstancedWEBGL(GLenum mode, const GLsizei* counts, GLenum type, const GLvoid* const* offsets, const GLsizei* instanceCounts, GLsizei drawcount);
+#endif /* EMSCRIPTEN_GL_WEBGL_multi_draw */
 
 // 44. https://www.khronos.org/registry/webgl/extensions/EXT_texture_norm16/
+#ifndef EMSCRIPTEN_GL_EXT_texture_norm16
+#define EMSCRIPTEN_GL_EXT_texture_norm16 1
 #define GL_R16_EXT 0x822A
 #define GL_RG16_EXT 0x822C
 #define GL_RGB16_EXT 0x8054
@@ -247,3 +338,4 @@ WEBGL_APICALL void GL_APIENTRY glMultiDrawElementsInstancedWEBGL(GLenum mode, co
 #define GL_RG16_SNORM_EXT 0x8F99
 #define GL_RGB16_SNORM_EXT 0x8F9A
 #define GL_RGBA16_SNORM_EXT 0x8F9B
+#endif /* EMSCRIPTEN_GL_EXT_texture_norm16 */
