@@ -1,3 +1,5 @@
+var hasModule = typeof Module === 'object' && Module;
+
 /** @param {boolean=} sync
     @param {number=} port */
 function reportResultToServer(result, sync, port) {
@@ -7,10 +9,10 @@ function reportResultToServer(result, sync, port) {
     reportErrorToServer("excessive reported results, sending " + result + ", test will fail");
   }
   reportResultToServer.reported = true;
-
   var xhr = new XMLHttpRequest();
-  var hasModule = typeof Module === 'object' && Module;
-  if (hasModule && Module['pageThrewException']) result = 12345;
+  if (hasModule && Module['pageThrewException']) {
+    result = 'pageThrewException';
+  }
   xhr.open('GET', 'http://localhost:' + port + '/report_result?' + result, !sync);
   xhr.send();
   if (typeof window === 'object' && window && hasModule && !Module['pageThrewException'] /* for easy debugging, don't close window on failure */) setTimeout(function() { window.close() }, 1000);
@@ -36,3 +38,13 @@ if (typeof window === 'object' && window) {
     xhr.send();
   });
 }
+
+if (hasModule) {
+  Module['onExit'] = function(status) {
+    maybeReportResultToServer('exit:' + status);
+  }
+
+  Module['onAbort'] = function(reason) {
+    maybeReportResultToServer('abort:' + reason);
+  }
+}
