@@ -9,30 +9,29 @@ import shutil
 import time
 import re
 import tempfile
-import zipfile
 from subprocess import PIPE, STDOUT
 
-from runner import RunnerCore, path_from_root, env_modify
-from runner import create_test_file, ensure_dir, make_executable
-from tools.config import config_file, EM_CONFIG
-from tools.shared import PYTHON, EMCC
+from runner import RunnerCore, path_from_root, env_modify, test_file
+from runner import create_file, ensure_dir, make_executable, with_env_modify
+from tools.config import EM_CONFIG
+from tools.shared import EMCC
 from tools.shared import CANONICAL_TEMP_DIR
 from tools.shared import try_delete, config
 from tools.shared import EXPECTED_LLVM_VERSION, Cache
 from tools import shared, system_libs, utils
 
 SANITY_FILE = shared.Cache.get_path('sanity.txt')
-commands = [[EMCC], [PYTHON, path_from_root('tests', 'runner.py'), 'blahblah']]
+commands = [[EMCC], [path_from_root('tests', 'runner'), 'blahblah']]
 
 
 def restore():
-  shutil.copyfile(config_file + '_backup', config_file)
+  shutil.copyfile(EM_CONFIG + '_backup', EM_CONFIG)
 
 
 # restore the config file and set it up for our uses
 def restore_and_set_up():
   restore()
-  with open(config_file, 'a') as f:
+  with open(EM_CONFIG, 'a') as f:
     # make LLVM_ROOT sensitive to the LLVM env var, as we test that
     f.write('LLVM_ROOT = "%s"\n' % config.LLVM_ROOT)
     # unfreeze the cache, so we can test that
@@ -41,12 +40,12 @@ def restore_and_set_up():
 
 # wipe the config and sanity files, creating a blank slate
 def wipe():
-  try_delete(config_file)
+  try_delete(EM_CONFIG)
   try_delete(SANITY_FILE)
 
 
 def add_to_config(content):
-  with open(config_file, 'a') as f:
+  with open(EM_CONFIG, 'a') as f:
     f.write('\n' + content + '\n')
 
 
@@ -100,7 +99,7 @@ EMBUILDER = path_from_root('embuilder.py')
 #  FILESYSTEM=0 avoids bringing libc for that)
 # (ERROR_ON_UNDEFINED_SYMBOLS=0 is needed because __errno_location is
 #  not included on the native side but needed by a lot of JS libraries.)
-MINIMAL_HELLO_WORLD = [path_from_root('tests', 'hello_world_em_asm.c'), '-O1', '-s', 'FILESYSTEM=0', '-s', 'ERROR_ON_UNDEFINED_SYMBOLS=0']
+MINIMAL_HELLO_WORLD = [test_file('hello_world_em_asm.c'), '-O1', '-s', 'FILESYSTEM=0', '-s', 'ERROR_ON_UNDEFINED_SYMBOLS=0']
 
 
 class sanity(RunnerCore):
@@ -111,15 +110,15 @@ class sanity(RunnerCore):
     # the sanity checks here
     del os.environ['EMCC_SKIP_SANITY_CHECK']
 
-    assert os.path.exists(config_file), 'To run these tests, we need a (working!) %s file to already exist' % EM_CONFIG
-    shutil.copyfile(config_file, config_file + '_backup')
+    assert os.path.exists(EM_CONFIG), 'To run these tests, we need a (working!) %s file to already exist' % EM_CONFIG
+    shutil.copyfile(EM_CONFIG, EM_CONFIG + '_backup')
 
     print()
     print('Running sanity checks.')
-    print('WARNING: This will modify %s, and in theory can break it although it should be restored properly. A backup will be saved in %s_backup' % (config_file, config_file))
+    print('WARNING: This will modify %s, and in theory can break it although it should be restored properly. A backup will be saved in %s_backup' % (EM_CONFIG, EM_CONFIG))
     print()
     print('>>> the original settings file is:')
-    print(open(config_file).read().strip())
+    print(open(EM_CONFIG).read().strip())
     print('<<<')
     print()
 
@@ -166,28 +165,29 @@ class sanity(RunnerCore):
       restore_and_set_up()
       self.check_working(command)
 
+  @with_env_modify({'EM_CONFIG': None})
   def test_firstrun(self):
     for command in commands:
       wipe()
 
-      def make_new_executable(name):
-        open(os.path.join(temp_bin, name), 'w').close()
-        make_executable(os.path.join(temp_bin, name))
-
-      env = os.environ.copy()
-      if 'EM_CONFIG' in env:
-        del env['EM_CONFIG']
+      default_config = config.embedded_config
 
       try:
         temp_bin = tempfile.mkdtemp()
+
+        def make_new_executable(name):
+          open(os.path.join(temp_bin, name), 'w').close()
+          make_executable(os.path.join(temp_bin, name))
+
         make_new_executable('llvm-dis')
         make_new_executable('node')
-        env['PATH'] = temp_bin + os.pathsep + os.environ['PATH']
-        output = self.do(command, env=env)
+        with env_modify({'PATH': temp_bin + os.pathsep + os.environ['PATH']}):
+          output = self.do(command)
       finally:
         shutil.rmtree(temp_bin)
+        config_data = open(default_config).read()
+        try_delete(default_config)
 
-      default_config = config.embedded_config
       self.assertContained('Welcome to Emscripten!', output)
       self.assertContained('This is the first time any of the Emscripten tools has been run.', output)
       self.assertContained('A settings file has been copied to %s, at absolute path: %s' % (default_config, default_config), output)
@@ -203,34 +203,34 @@ class sanity(RunnerCore):
         self.assertIdentical(possible_nodes, re.search("^ *NODE_JS *= (.*)$", output, re.M).group(1))
       self.assertContained('Please edit the file if any of those are incorrect', output)
       self.assertContained('This command will now exit. When you are done editing those paths, re-run it.', output)
-      assert output.split()[-1].endswith('===='), 'We should have stopped: ' + output
-      config_file = open(default_config).read()
+      self.assertTrue(output.strip().endswith('============='))
       template_file = open(path_from_root('tools', 'settings_template.py')).read()
-      self.assertNotContained('{{{', config_file)
-      self.assertNotContained('}}}', config_file)
+      self.assertNotContained('{{{', config_data)
+      self.assertNotContained('}}}', config_data)
       self.assertContained('{{{', template_file)
       self.assertContained('}}}', template_file)
       for content in ['EMSCRIPTEN_ROOT', 'LLVM_ROOT', 'NODE_JS', 'JS_ENGINES']:
-        self.assertContained(content, config_file)
+        self.assertContained(content, config_data)
 
       # The guessed config should be ok
       # XXX This depends on your local system! it is possible `which` guesses wrong
       # try_delete('a.out.js')
-      # output = self.run_process([EMCC, path_from_root('tests', 'hello_world.c')], stdout=PIPE, stderr=PIPE).output
+      # output = self.run_process([EMCC, test_file('hello_world.c')], stdout=PIPE, stderr=PIPE).output
       # self.assertContained('hello, world!', self.run_js('a.out.js'), output)
 
       # Second run, with bad EM_CONFIG
       for settings in ['blah', 'LLVM_ROOT="blarg"; JS_ENGINES=[]; NODE_JS=[]; SPIDERMONKEY_ENGINE=[]']:
-        f = open(default_config, 'w')
-        f.write(settings)
-        f.close()
-        output = self.do(command, env=env)
+        try:
+          with open(default_config, 'w') as f:
+            f.write(settings)
+          output = self.do(command)
 
-        if 'LLVM_ROOT' not in settings:
-          self.assertContained('Error in evaluating %s' % default_config, output)
-        elif 'runner.py' not in ' '.join(command):
-          self.assertContained('error:', output) # sanity check should fail
-      try_delete(default_config)
+          if 'blah' in settings:
+            self.assertContained('Error in evaluating config file (%s)' % default_config, output)
+          elif 'runner' not in ' '.join(command):
+            self.assertContained('error: NODE_JS is set to empty value', output) # sanity check should fail
+        finally:
+          try_delete(default_config)
 
   def test_llvm(self):
     LLVM_WARNING = 'LLVM version appears incorrect'
@@ -244,7 +244,7 @@ class sanity(RunnerCore):
 
     # Fake a different llvm version
     restore_and_set_up()
-    with open(config_file, 'a') as f:
+    with open(EM_CONFIG, 'a') as f:
       f.write('LLVM_ROOT = "' + self.in_dir('fake') + '"')
 
     real_version_x, real_version_y = (int(x) for x in EXPECTED_LLVM_VERSION.split('.'))
@@ -293,7 +293,7 @@ class sanity(RunnerCore):
 
     # Fake a different node version
     restore_and_set_up()
-    with open(config_file, 'a') as f:
+    with open(EM_CONFIG, 'a') as f:
       f.write('NODE_JS = "' + self.in_dir('fake', 'nodejs') + '"')
 
     ensure_dir('fake')
@@ -367,7 +367,7 @@ fi
     self.assertNotContained(SANITY_FAIL_MESSAGE, output)
 
     # emcc should also check sanity if the file is outdated
-    open(config_file, 'a').write('# extra stuff\n')
+    open(EM_CONFIG, 'a').write('# extra stuff\n')
     output = self.check_working(EMCC)
     self.assertContained(SANITY_MESSAGE, output)
     self.assertNotContained(SANITY_FAIL_MESSAGE, output)
@@ -375,7 +375,7 @@ fi
   def test_em_config_env_var(self):
     # emcc should be configurable directly from EM_CONFIG without any config file
     restore_and_set_up()
-    create_test_file('main.cpp', '''
+    create_file('main.cpp', '''
       #include <stdio.h>
       int main() {
         printf("hello from emcc with no config file\\n");
@@ -385,9 +385,9 @@ fi
 
     wipe()
     with env_modify({'EM_CONFIG': get_basic_config()}):
-      self.run_process([EMCC, 'main.cpp', '-Wno-deprecated', '-o', 'a.out.js'])
+      out = self.expect_fail([EMCC, 'main.cpp', '-Wno-deprecated', '-o', 'a.out.js'])
 
-    self.assertContained('hello from emcc with no config file', self.run_js('a.out.js'))
+    self.assertContained('error: Inline EM_CONFIG data no longer supported.  Please use a config file.', out)
 
   def clear_cache(self):
     self.run_process([EMCC, '--clear-cache'])
@@ -399,7 +399,7 @@ fi
       self.assertIn(os.listdir(Cache.dirname), ([], ['cache.lock']))
 
   def ensure_cache(self):
-    self.do([EMCC, '-O2', path_from_root('tests', 'hello_world.c')])
+    self.do([EMCC, '-O2', test_file('hello_world.c')])
 
   def test_emcc_caching(self):
     BUILDING_MESSAGE = 'generating system library: %s'
@@ -413,7 +413,7 @@ fi
     for i in range(3):
       print(i)
       self.clear()
-      output = self.do([EMCC, '-O' + str(i), path_from_root('tests', 'hello_libcxx.cpp'), '-s', 'DISABLE_EXCEPTION_CATCHING=0'])
+      output = self.do([EMCC, '-O' + str(i), test_file('hello_libcxx.cpp'), '-s', 'DISABLE_EXCEPTION_CATCHING=0'])
       print('\n\n\n', output)
       self.assertContainedIf(BUILDING_MESSAGE % libname, output, i == 0)
       self.assertContained('hello, world!', self.run_js('a.out.js'))
@@ -462,7 +462,7 @@ fi
   def test_emcc_multiprocess_cache_access(self):
     restore_and_set_up()
 
-    create_test_file('test.c', r'''
+    create_file('test.c', r'''
       #include <stdio.h>
       int main() {
         printf("hello, world!\n");
@@ -494,14 +494,14 @@ fi
 
     fd, custom_config_filename = tempfile.mkstemp(prefix='.emscripten_config_')
 
-    orig_config = open(config_file, 'r').read()
+    orig_config = open(EM_CONFIG, 'r').read()
 
     # Move the ~/.emscripten to a custom location.
     with os.fdopen(fd, "w") as f:
       f.write(get_basic_config())
 
     # Make a syntax error in the original config file so that attempting to access it would fail.
-    open(config_file, 'w').write('asdfasdfasdfasdf\n\'\'\'' + orig_config)
+    open(EM_CONFIG, 'w').write('asdfasdfasdfasdf\n\'\'\'' + orig_config)
 
     temp_dir = tempfile.mkdtemp(prefix='emscripten_temp_')
 
@@ -541,22 +541,20 @@ fi
       self.assertNotExists(PORTS_DIR)
 
       # Building a file that doesn't need ports should not trigger anything
-      output = self.do([EMCC, path_from_root('tests', 'hello_world_sdl.cpp')])
-      assert RETRIEVING_MESSAGE not in output, output
-      assert BUILDING_MESSAGE not in output
-      print('no', output)
+      output = self.do([EMCC, test_file('hello_world_sdl.cpp')])
+      self.assertNotContained(RETRIEVING_MESSAGE, output)
+      self.assertNotContained(BUILDING_MESSAGE, output)
       self.assertNotExists(PORTS_DIR)
 
       def first_use():
-        output = self.do([EMCC, path_from_root('tests', 'hello_world_sdl.cpp'), '-s', 'USE_SDL=2'])
+        output = self.do([EMCC, test_file('hello_world_sdl.cpp'), '-s', 'USE_SDL=2'])
         self.assertContained(RETRIEVING_MESSAGE, output)
         self.assertContained(BUILDING_MESSAGE, output)
         self.assertExists(PORTS_DIR)
-        print('yes', output)
 
       def second_use():
         # Using it again avoids retrieve and build
-        output = self.do([EMCC, path_from_root('tests', 'hello_world_sdl.cpp'), '-s', 'USE_SDL=2'])
+        output = self.do([EMCC, test_file('hello_world_sdl.cpp'), '-s', 'USE_SDL=2'])
         self.assertNotContained(RETRIEVING_MESSAGE, output)
         self.assertNotContained(BUILDING_MESSAGE, output)
 
@@ -564,15 +562,9 @@ fi
       first_use()
       second_use()
 
-      # if the tag doesn't match, we retrieve and rebuild
-      subdir = os.listdir(os.path.join(PORTS_DIR, 'sdl2'))[0]
-      os.rename(os.path.join(PORTS_DIR, 'sdl2', subdir), os.path.join(PORTS_DIR, 'sdl2', 'old-subdir'))
-      ensure_dir('old-sub')
-      open(os.path.join('old-sub', 'a.txt'), 'w').write('waka')
-      open(os.path.join('old-sub', 'b.txt'), 'w').write('waka')
-      with zipfile.ZipFile(os.path.join(PORTS_DIR, 'sdl2.zip'), 'w') as z:
-        z.write(os.path.join('old-sub', 'a.txt'))
-        z.write(os.path.join('old-sub', 'b.txt'))
+      # if the url doesn't match, we retrieve and rebuild
+      with open(os.path.join(PORTS_DIR, 'sdl2', '.emscripten_url'), 'w') as f:
+        f.write('foo')
 
       first_use()
       second_use()
@@ -581,7 +573,7 @@ fi
     # Test that running JS commands works for node, d8, and jsc and is not path dependent
     restore_and_set_up()
 
-    sample_script = path_from_root('tests', 'print_args.js')
+    sample_script = test_file('print_args.js')
 
     # Fake some JS engines
     # Note that the path contains 'd8'.
@@ -639,7 +631,7 @@ fi
     self.clear_cache()
 
     def make_fake(report):
-      with open(config_file, 'a') as f:
+      with open(EM_CONFIG, 'a') as f:
         f.write('LLVM_ROOT = "' + self.in_dir('fake', 'bin') + '"\n')
         # BINARYEN_ROOT needs to exist in the config, even though this test
         # doesn't actually use it.
@@ -663,11 +655,11 @@ fi
     # with no binaryen root, an error is shown
     restore_and_set_up()
 
-    open(config_file, 'a').write('\nBINARYEN_ROOT = ""\n')
-    self.check_working([EMCC, path_from_root('tests', 'hello_world.c')], 'BINARYEN_ROOT is set to empty value in %s' % config_file)
+    open(EM_CONFIG, 'a').write('\nBINARYEN_ROOT = ""\n')
+    self.check_working([EMCC, test_file('hello_world.c')], 'BINARYEN_ROOT is set to empty value in %s' % EM_CONFIG)
 
-    open(config_file, 'a').write('\ndel BINARYEN_ROOT\n')
-    self.check_working([EMCC, path_from_root('tests', 'hello_world.c')], 'BINARYEN_ROOT is not defined in %s' % config_file)
+    open(EM_CONFIG, 'a').write('\ndel BINARYEN_ROOT\n')
+    self.check_working([EMCC, test_file('hello_world.c')], 'BINARYEN_ROOT is not defined in %s' % EM_CONFIG)
 
   def test_embuilder_force(self):
     restore_and_set_up()
@@ -697,11 +689,11 @@ fi
 
   def test_binaryen_version(self):
     restore_and_set_up()
-    with open(config_file, 'a') as f:
+    with open(EM_CONFIG, 'a') as f:
       f.write('\nBINARYEN_ROOT = "' + self.in_dir('fake') + '"')
 
     make_fake_tool(self.in_dir('fake', 'bin', 'wasm-opt'), 'foo')
-    self.check_working([EMCC, path_from_root('tests', 'hello_world.c')], 'error parsing binaryen version (wasm-opt version foo). Please check your binaryen installation')
+    self.check_working([EMCC, test_file('hello_world.c')], 'error parsing binaryen version (wasm-opt version foo). Please check your binaryen installation')
 
     make_fake_tool(self.in_dir('fake', 'bin', 'wasm-opt'), '70')
-    self.check_working([EMCC, path_from_root('tests', 'hello_world.c')], 'unexpected binaryen version: 70 (expected ')
+    self.check_working([EMCC, test_file('hello_world.c')], 'unexpected binaryen version: 70 (expected ')
