@@ -10,8 +10,6 @@ v128_t TESTFN i8x16_load(void *ptr) {
   return wasm_v128_load(ptr);
 }
 
-#ifdef __wasm_unimplemented_simd128__
-
 v128_t TESTFN v8x16_load_splat(void *ptr) {
   return wasm_v8x16_load_splat(ptr);
 }
@@ -43,8 +41,6 @@ v128_t TESTFN i64x2_load32x2_u(void *ptr) {
   return wasm_u64x2_load_32x2(ptr);
 }
 
-#endif // __wasm_unimplemented_simd128__
-
 void TESTFN i8x16_store(void *ptr, v128_t vec) {
   wasm_v128_store(ptr, vec);
 }
@@ -106,14 +102,10 @@ v128_t TESTFN i64x2_shuffle_reverse(v128_t vec) {
   return wasm_v64x2_shuffle(vec, vec, 1, 0);
 }
 
-#ifdef __wasm_unimplemented_simd128__
-
 v128_t TESTFN v8x16_swizzle(v128_t x, v128_t y) {
   return wasm_v8x16_swizzle(x, y);
 }
 
-#endif // __wasm_unimplemented_simd128__
-
 v128_t TESTFN i8x16_splat(int32_t x) {
   return wasm_i8x16_splat(x);
 }
@@ -363,23 +355,22 @@ v128_t TESTFN v128_xor(v128_t x, v128_t y) {
   return wasm_v128_xor(x, y);
 }
 
-#ifdef __wasm_unimplemented_simd128__
-
 v128_t TESTFN v128_andnot(v128_t x, v128_t y) {
   return wasm_v128_andnot(x, y);
 }
 
-#endif // __wasm_unimplemented_simd128__
-
 v128_t TESTFN v128_bitselect(v128_t x, v128_t y, v128_t cond) {
   return wasm_v128_bitselect(x, y, cond);
 }
+v128_t TESTFN i8x16_abs(v128_t vec) {
+  return wasm_i8x16_abs(vec);
+}
 v128_t TESTFN i8x16_neg(v128_t vec) {
   return wasm_i8x16_neg(vec);
 }
-int32_t TESTFN i8x16_any_true(v128_t vec) {
-  return wasm_i8x16_any_true(vec);
-}
+/* int32_t TESTFN i8x16_any_true(v128_t vec) { */
+/*   return wasm_i8x16_any_true(vec); */
+/* } */
 int32_t TESTFN i8x16_all_true(v128_t vec) {
   return wasm_i8x16_all_true(vec);
 }
@@ -410,30 +401,27 @@ v128_t TESTFN i8x16_sub_saturate_s(v128_t x, v128_t y) {
 v128_t TESTFN i8x16_sub_saturate_u(v128_t x, v128_t y) {
   return wasm_u8x16_sub_saturate(x, y);
 }
-v128_t TESTFN i8x16_mul(v128_t x, v128_t y) {
-  return wasm_i8x16_mul(x, y);
-}
 v128_t TESTFN i8x16_min_s(v128_t x, v128_t y) {
-  return wasm_i8x16_min_s(x, y);
+  return wasm_i8x16_min(x, y);
 }
 v128_t TESTFN i8x16_min_u(v128_t x, v128_t y) {
-  return wasm_i8x16_min_u(x, y);
+  return wasm_u8x16_min(x, y);
 }
 v128_t TESTFN i8x16_max_s(v128_t x, v128_t y) {
-  return wasm_i8x16_max_s(x, y);
+  return wasm_i8x16_max(x, y);
 }
 v128_t TESTFN i8x16_max_u(v128_t x, v128_t y) {
-  return wasm_i8x16_max_u(x, y);
+  return wasm_u8x16_max(x, y);
 }
 v128_t TESTFN i8x16_avgr_u(v128_t x, v128_t y) {
-  return wasm_i8x16_avgr_u(x, y);
+  return wasm_u8x16_avgr(x, y);
+}
+v128_t TESTFN i16x8_abs(v128_t vec) {
+  return wasm_i16x8_abs(vec);
 }
 v128_t TESTFN i16x8_neg(v128_t vec) {
   return wasm_i16x8_neg(vec);
 }
-bool TESTFN i16x8_any_true(v128_t vec) {
-  return wasm_i16x8_any_true(vec);
-}
 bool TESTFN i16x8_all_true(v128_t vec) {
   return wasm_i16x8_all_true(vec);
 }
@@ -465,29 +453,29 @@ v128_t TESTFN i16x8_sub_saturate_u(v128_t x, v128_t y) {
   return wasm_u16x8_sub_saturate(x, y);
 }
 v128_t TESTFN i16x8_min_s(v128_t x, v128_t y) {
-  return wasm_i16x8_min_s(x, y);
+  return wasm_i16x8_min(x, y);
 }
 v128_t TESTFN i16x8_min_u(v128_t x, v128_t y) {
-  return wasm_i16x8_min_u(x, y);
+  return wasm_u16x8_min(x, y);
 }
 v128_t TESTFN i16x8_max_s(v128_t x, v128_t y) {
-  return wasm_i16x8_max_s(x, y);
+  return wasm_i16x8_max(x, y);
 }
 v128_t TESTFN i16x8_max_u(v128_t x, v128_t y) {
-  return wasm_i16x8_max_u(x, y);
+  return wasm_u16x8_max(x, y);
 }
 v128_t TESTFN i16x8_avgr_u(v128_t x, v128_t y) {
-  return wasm_i16x8_avgr_u(x, y);
+  return wasm_u16x8_avgr(x, y);
 }
 v128_t TESTFN i16x8_mul(v128_t x, v128_t y) {
   return wasm_i16x8_mul(x, y);
 }
+v128_t TESTFN i32x4_abs(v128_t vec) {
+  return wasm_i32x4_abs(vec);
+}
 v128_t TESTFN i32x4_neg(v128_t vec) {
   return wasm_i32x4_neg(vec);
 }
-int32_t TESTFN i32x4_any_true(v128_t vec) {
-  return wasm_i32x4_any_true(vec);
-}
 int32_t TESTFN i32x4_all_true(v128_t vec) {
   return wasm_i32x4_all_true(vec);
 }
@@ -507,16 +495,16 @@ v128_t TESTFN i32x4_sub(v128_t x, v128_t y) {
   return wasm_i32x4_sub(x, y);
 }
 v128_t TESTFN i32x4_min_s(v128_t x, v128_t y) {
-  return wasm_i32x4_min_s(x, y);
+  return wasm_i32x4_min(x, y);
 }
 v128_t TESTFN i32x4_min_u(v128_t x, v128_t y) {
-  return wasm_i32x4_min_u(x, y);
+  return wasm_u32x4_min(x, y);
 }
 v128_t TESTFN i32x4_max_s(v128_t x, v128_t y) {
-  return wasm_i32x4_max_s(x, y);
+  return wasm_i32x4_max(x, y);
 }
 v128_t TESTFN i32x4_max_u(v128_t x, v128_t y) {
-  return wasm_i32x4_max_u(x, y);
+  return wasm_u32x4_max(x, y);
 }
 v128_t TESTFN i32x4_mul(v128_t x, v128_t y) {
   return wasm_i32x4_mul(x, y);
@@ -524,18 +512,6 @@ v128_t TESTFN i32x4_mul(v128_t x, v128_t y) {
 v128_t TESTFN i64x2_neg(v128_t vec) {
   return wasm_i64x2_neg(vec);
 }
-
-#ifdef __wasm_unimplemented_simd128__
-
-bool TESTFN i64x2_any_true(v128_t vec) {
-  return wasm_i64x2_any_true(vec);
-}
-bool TESTFN i64x2_all_true(v128_t vec) {
-  return wasm_i64x2_all_true(vec);
-}
-
-#endif // __wasm_unimplemented_simd128__
-
 v128_t TESTFN i64x2_shl(v128_t vec, int32_t shift) {
   return wasm_i64x2_shl(vec, shift);
 }
@@ -560,18 +536,6 @@ v128_t TESTFN f32x4_neg(v128_t vec) {
 v128_t TESTFN f32x4_sqrt(v128_t vec) {
   return wasm_f32x4_sqrt(vec);
 }
-
-#ifdef __wasm_unimplemented_simd128__
-
-v128_t TESTFN f32x4_qfma(v128_t a, v128_t b, v128_t c) {
-  return wasm_f32x4_qfma(a, b, c);
-}
-v128_t TESTFN f32x4_qfms(v128_t a, v128_t b, v128_t c) {
-  return wasm_f32x4_qfms(a, b, c);
-}
-
-#endif // __wasm_unimplemented_simd128__
-
 v128_t TESTFN f32x4_add(v128_t x, v128_t y) {
   return wasm_f32x4_add(x, y);
 }
@@ -599,18 +563,6 @@ v128_t TESTFN f64x2_neg(v128_t vec) {
 v128_t TESTFN f64x2_sqrt(v128_t vec) {
   return wasm_f64x2_sqrt(vec);
 }
-
-#ifdef __wasm_unimplemented_simd128__
-
-v128_t TESTFN f64x2_qfma(v128_t a, v128_t b, v128_t c) {
-  return wasm_f64x2_qfma(a, b, c);
-}
-v128_t TESTFN f64x2_qfms(v128_t a, v128_t b, v128_t c) {
-  return wasm_f64x2_qfms(a, b, c);
-}
-
-#endif // __wasm_unimplemented_simd128__
-
 v128_t TESTFN f64x2_add(v128_t x, v128_t y) {
   return wasm_f64x2_add(x, y);
 }
@@ -636,17 +588,6 @@ v128_t TESTFN i32x4_trunc_u_f32x4_sat(v128_t vec) {
   return wasm_u32x4_trunc_saturate_f32x4(vec);
 }
 
-#ifdef __wasm_unimplemented_simd128__
-
-v128_t TESTFN i64x2_trunc_s_f64x2_sat(v128_t vec) {
-  return wasm_i64x2_trunc_saturate_f64x2(vec);
-}
-v128_t TESTFN i64x2_trunc_u_f64x2_sat(v128_t vec) {
-  return wasm_u64x2_trunc_saturate_f64x2(vec);
-}
-
-#endif // __wasm_unimplemented_simd128__
-
 v128_t TESTFN f32x4_convert_s_i32x4(v128_t vec) {
   return wasm_f32x4_convert_i32x4(vec);
 }
@@ -654,17 +595,6 @@ v128_t TESTFN f32x4_convert_u_i32x4(v128_t vec) {
   return wasm_f32x4_convert_u32x4(vec);
 }
 
-#ifdef __wasm_unimplemented_simd128__
-
-v128_t TESTFN f64x2_convert_s_i64x2(v128_t vec) {
-  return wasm_f64x2_convert_i64x2(vec);
-}
-v128_t TESTFN f64x2_convert_u_i64x2(v128_t vec) {
-  return wasm_f64x2_convert_u64x2(vec);
-}
-
-#endif // __wasm_unimplemented_simd128__
-
 v128_t TESTFN i8x16_narrow_i16x8_s(v128_t a, v128_t b) {
   return wasm_i8x16_narrow_i16x8(a, b);
 }
@@ -817,8 +747,6 @@ int EMSCRIPTEN_KEEPALIVE __attribute__((__optnone__)) main(int argc, char** argv
                i8x16(7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7));
   }
 
-#ifdef __wasm_unimplemented_simd128__
-
   {
     char vec[8] = {0x80, 0x90, 0xa0, 0xb0, 0xc0, 0xd0, 0xe0, 0xf0};
     void *ptr = &vec;
@@ -859,8 +787,6 @@ int EMSCRIPTEN_KEEPALIVE __attribute__((__optnone__)) main(int argc, char** argv
     );
   }
 
-#endif // __wasm_unimplemented_simd128__
-
   expect_vec(i8x16_const(), u8x16(1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16));
   expect_vec(i16x8_const(), u16x8(1, 2, 3, 4, 5, 6, 7, 8));
   expect_vec(i32x4_const(), u32x4(1, 2, 3, 4));
@@ -889,8 +815,6 @@ int EMSCRIPTEN_KEEPALIVE __attribute__((__optnone__)) main(int argc, char** argv
   expect_vec(i32x4_shuffle_reverse((v128_t)i32x4(1, 2, 3, 4)), i32x4(4, 3, 2, 1));
   expect_vec(i64x2_shuffle_reverse((v128_t)i64x2(1, 2)), i64x2(2, 1));
 
-#ifdef  __wasm_unimplemented_simd128__
-
   expect_vec(
     v8x16_swizzle(
       (v128_t)i8x16(0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7,
@@ -901,8 +825,6 @@ int EMSCRIPTEN_KEEPALIVE __attribute__((__optnone__)) main(int argc, char** argv
           0x00, 0x00, 0xff, 0xfd, 0xfc, 0xf8, 0xf4, 0xf0)
   );
 
-#endif // __wasm_unimplemented_simd128__
-
   // i8x16 lane accesses
   expect_vec(i8x16_splat(5), i8x16(5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5));
   expect_vec(i8x16_splat(257), i8x16(1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1));
@@ -1282,16 +1204,10 @@ int EMSCRIPTEN_KEEPALIVE __attribute__((__optnone__)) main(int argc, char** argv
     v128_xor((v128_t)i32x4(0, 0, -1, -1), (v128_t)i32x4(0, -1, 0, -1)),
     i32x4(0, -1, -1, 0)
   );
-
-#ifdef __wasm_unimplemented_simd128__
-
   expect_vec(
     v128_andnot((v128_t)i32x4(0, 0, -1, -1), (v128_t)i32x4(0, -1, 0, -1)),
-    i32x4(0, -1, -1, 0)
+    i32x4(0, 0, -1, 0)
   );
-
-#endif // __wasm_unimplemented_simd128__
-
   expect_vec(
     v128_bitselect(
       (v128_t)i32x4(0xAAAAAAAA, 0xAAAAAAAA, 0xAAAAAAAA, 0xAAAAAAAA),
@@ -1302,14 +1218,18 @@ int EMSCRIPTEN_KEEPALIVE __attribute__((__optnone__)) main(int argc, char** argv
   );
 
   // i8x16 arithmetic
+  expect_vec(
+    i8x16_abs((v128_t)i8x16(0, 1, 42, -3, -56, 127, -128, -126, 0, -1, -42, 3, 56, -127, -128, 126)),
+    i8x16(0, 1, 42, 3, 56, 127, -128, 126, 0, 1, 42, 3, 56, 127, -128, 126)
+  );
   expect_vec(
     i8x16_neg((v128_t)i8x16(0, 1, 42, -3, -56, 127, -128, -126, 0, -1, -42, 3, 56, -127, -128, 126)),
     i8x16(0, -1, -42, 3, 56, -127, -128, 126, 0, 1, 42, -3, -56, 127, -128, -126)
   );
-  expect_eq(i8x16_any_true((v128_t)i8x16(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)), 0);
-  expect_eq(i8x16_any_true((v128_t)i8x16(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0)), 1);
-  expect_eq(i8x16_any_true((v128_t)i8x16(1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1)), 1);
-  expect_eq(i8x16_any_true((v128_t)i8x16(1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1)), 1);
+  /* expect_eq(i8x16_any_true((v128_t)i8x16(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)), 0); */
+  /* expect_eq(i8x16_any_true((v128_t)i8x16(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0)), 1); */
+  /* expect_eq(i8x16_any_true((v128_t)i8x16(1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1)), 1); */
+  /* expect_eq(i8x16_any_true((v128_t)i8x16(1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1)), 1); */
   expect_eq(i8x16_all_true((v128_t)i8x16(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)), 0);
   expect_eq(i8x16_all_true((v128_t)i8x16(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0)), 0);
   expect_eq(i8x16_all_true((v128_t)i8x16(1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1)), 0);
@@ -1380,13 +1300,6 @@ int EMSCRIPTEN_KEEPALIVE __attribute__((__optnone__)) main(int argc, char** argv
     ),
     u8x16(0, 0, 254, 0, 0, 123, 0, 12, 61, 167, 158, 100, 17, 0, 0, 0)
   );
-  expect_vec(
-    i8x16_mul(
-      (v128_t)i8x16(0, 42, 255, 128, 127, 129, 6, 29, 103, 196, 231, 142, 17, 250, 1, 73),
-      (v128_t)i8x16(3, 231, 1, 128, 129, 6, 103, 17, 42, 29, 73, 42, 0, 255, 127, 142)
-    ),
-    i8x16(0, 230, 255, 0, 255, 6, 106, 237, 230, 52, 223, 76, 0, 6, 127, 126)
-  );
   expect_vec(
     i8x16_min_s(
       (v128_t)i8x16(0, 42, 255, 128, 127, 129, 6, 29, 103, 196, 231, 142, 17, 250, 1, 73),
@@ -1424,14 +1337,14 @@ int EMSCRIPTEN_KEEPALIVE __attribute__((__optnone__)) main(int argc, char** argv
   );
 
   // i16x8 arithmetic
+  expect_vec(
+    i16x8_abs((v128_t)i16x8(0, 1, 42, -3, -56, 32767, -32768, 32766)),
+    i16x8(0, 1, 42, 3, 56, 32767, -32768, 32766)
+  );
   expect_vec(
     i16x8_neg((v128_t)i16x8(0, 1, 42, -3, -56, 32767, -32768, 32766)),
     i16x8(0, -1, -42, 3, 56, -32767, -32768, -32766)
   );
-  expect_eq(i16x8_any_true((v128_t)i16x8(0, 0, 0, 0, 0, 0, 0, 0)), 0);
-  expect_eq(i16x8_any_true((v128_t)i16x8(0, 0, 1, 0, 0, 0, 0, 0)), 1);
-  expect_eq(i16x8_any_true((v128_t)i16x8(1, 1, 1, 1, 1, 0, 1, 1)), 1);
-  expect_eq(i16x8_any_true((v128_t)i16x8(1, 1, 1, 1, 1, 1, 1, 1)), 1);
   expect_eq(i16x8_all_true((v128_t)i16x8(0, 0, 0, 0, 0, 0, 0, 0)), 0);
   expect_eq(i16x8_all_true((v128_t)i16x8(0, 0, 1, 0, 0, 0, 0, 0)), 0);
   expect_eq(i16x8_all_true((v128_t)i16x8(1, 1, 1, 1, 1, 0, 1, 1)), 0);
@@ -1547,13 +1460,13 @@ int EMSCRIPTEN_KEEPALIVE __attribute__((__optnone__)) main(int argc, char** argv
 
   // i32x4 arithmetic
   expect_vec(
-    i32x4_neg((v128_t)i32x4(0, 1, 0x80000000, 0x7fffffff)),
-    i32x4(0, -1, 0x80000000, 0x80000001)
+    i32x4_abs((v128_t)i32x4(0, 1, 0x80000000, 0x80000001)),
+    i32x4(0, 1, 0x80000000, 0x7fffffff)
+  );
+  expect_vec(
+    i32x4_neg((v128_t)i32x4(0, 1, 0x80000000, 0x80000001)),
+    i32x4(0, -1, 0x80000000, 0x7fffffff)
   );
-  expect_eq(i32x4_any_true((v128_t)i32x4(0, 0, 0, 0)), 0);
-  expect_eq(i32x4_any_true((v128_t)i32x4(0, 0, 1, 0)), 1);
-  expect_eq(i32x4_any_true((v128_t)i32x4(1, 0, 1, 1)), 1);
-  expect_eq(i32x4_any_true((v128_t)i32x4(1, 1, 1, 1)), 1);
   expect_eq(i32x4_all_true((v128_t)i32x4(0, 0, 0, 0)), 0);
   expect_eq(i32x4_all_true((v128_t)i32x4(0, 0, 1, 0)), 0);
   expect_eq(i32x4_all_true((v128_t)i32x4(1, 0, 1, 1)), 0);
@@ -1613,12 +1526,6 @@ int EMSCRIPTEN_KEEPALIVE __attribute__((__optnone__)) main(int argc, char** argv
 
   // i64x2 arithmetic
   expect_vec(i64x2_neg((v128_t)i64x2(0x8000000000000000, 42)), i64x2(0x8000000000000000, -42));
-  /* expect_eq(i64x2_any_true((v128_t)i64x2(0, 0)), 0); */
-  /* expect_eq(i64x2_any_true((v128_t)i64x2(1, 0)), 1); */
-  /* expect_eq(i64x2_any_true((v128_t)i64x2(1, 1)), 1); */
-  /* expect_eq(i64x2_all_true((v128_t)i64x2(0, 0)), 0); */
-  /* expect_eq(i64x2_all_true((v128_t)i64x2(1, 0)), 0); */
-  /* expect_eq(i64x2_all_true((v128_t)i64x2(1, 1)), 1); */
 
   expect_vec(i64x2_shl((v128_t)i64x2(1, 0x8000000000000000), 1), i64x2(2, 0));
   expect_vec(i64x2_shl((v128_t)i64x2(1, 0x8000000000000000), 64), i64x2(1, 0x8000000000000000));
@@ -1639,7 +1546,6 @@ int EMSCRIPTEN_KEEPALIVE __attribute__((__optnone__)) main(int argc, char** argv
   expect_vec(f32x4_abs((v128_t)f32x4(-0., NAN, -INFINITY, 5)), f32x4(0, NAN, INFINITY, 5));
   expect_vec(f32x4_neg((v128_t)f32x4(-0., NAN, -INFINITY, 5)), f32x4(0, -NAN, INFINITY, -5));
   expect_vec(f32x4_sqrt((v128_t)f32x4(0., NAN, INFINITY, 4)), f32x4(-0., NAN, INFINITY, 2));
-  // TODO: test QFMA/QFMS
   expect_vec(
     f32x4_add((v128_t)f32x4(NAN, -NAN, INFINITY, 42), (v128_t)f32x4(42, INFINITY, INFINITY, 1)),
     f32x4(NAN, -NAN, INFINITY, 43)
@@ -1670,10 +1576,6 @@ int EMSCRIPTEN_KEEPALIVE __attribute__((__optnone__)) main(int argc, char** argv
   expect_vec(f64x2_abs((v128_t)f64x2(-INFINITY, 5)), f64x2(INFINITY, 5));
   expect_vec(f64x2_neg((v128_t)f64x2(-0., NAN)), f64x2(0, -NAN));
   expect_vec(f64x2_neg((v128_t)f64x2(-INFINITY, 5)), f64x2(INFINITY, -5));
-  // TODO: test QFMA/QFMS
-  // https://bugs.chromium.org/p/v8/issues/detail?id=10170
-  //  expect_vec(f64x2_sqrt((v128_t)f64x2(-0., NAN)), f64x2(-0., NAN));
-  //expect_vec(f64x2_sqrt((v128_t)f64x2(INFINITY, 4)), f64x2(INFINITY, 2));
   expect_vec(
     f64x2_add((v128_t)f64x2(NAN, -NAN), (v128_t)f64x2(42, INFINITY)),
     f64x2(NAN, -NAN)
@@ -1720,22 +1622,6 @@ int EMSCRIPTEN_KEEPALIVE __attribute__((__optnone__)) main(int argc, char** argv
     i32x4_trunc_u_f32x4_sat((v128_t)f32x4(42, NAN, INFINITY, -INFINITY)),
     u32x4(42, 0, 4294967295ull, 0)
   );
-
-#ifdef __wasm_unimplemented_simd128__
-
-  expect_vec(i64x2_trunc_s_f64x2_sat((v128_t)f64x2(42, NAN)), i64x2(42, 0));
-  expect_vec(
-    i64x2_trunc_s_f64x2_sat((v128_t)f64x2(INFINITY, -INFINITY)),
-    i64x2(9223372036854775807ll, -9223372036854775807ll - 1)
-  );
-  expect_vec(i64x2_trunc_u_f64x2_sat((v128_t)f64x2(42, NAN)), u64x2(42, 0));
-  expect_vec(
-    i64x2_trunc_u_f64x2_sat((v128_t)f64x2(INFINITY, -INFINITY)),
-    u64x2(18446744073709551615ull, 0)
-  );
-
-#endif // __wasm_unimplemented_simd128__
-
   expect_vec(
     f32x4_convert_s_i32x4((v128_t)i32x4(0, -1, 2147483647, -2147483647 - 1)),
     f32x4(0, -1, 2147483648., -2147483648.)
@@ -1744,25 +1630,6 @@ int EMSCRIPTEN_KEEPALIVE __attribute__((__optnone__)) main(int argc, char** argv
     f32x4_convert_u_i32x4((v128_t)u32x4(0, -1, 2147483647, -2147483647 - 1)),
     f32x4(0, 4294967296., 2147483648., 2147483648.)
   );
-
-#ifdef __wasm_unimplemented_simd128__
-
-  expect_vec(f64x2_convert_s_i64x2((v128_t)i64x2(0, -1)), f64x2(0, -1));
-  expect_vec(
-    f64x2_convert_s_i64x2((v128_t)i64x2(9223372036854775807, -9223372036854775807 - 1)),
-    f64x2(9223372036854775807., -9223372036854775808.)
-  );
-  expect_vec(
-    f64x2_convert_u_i64x2((v128_t)u64x2(0, -1)),
-    f64x2(0, 18446744073709551616.)
-  );
-  expect_vec(
-    f64x2_convert_u_i64x2((v128_t)i64x2(9223372036854775807 , -9223372036854775808.)),
-    f64x2(9223372036854775807., 9223372036854775808.)
-  );
-
-#endif // __wasm_unimplemented_simd128__
-
   expect_vec(
     i8x16_narrow_i16x8_s(
       (v128_t)i16x8(129, 127, -32767, 32767, -32768, -1, 1, 0),
