@@ -19,27 +19,24 @@ def get(ports, settings, shared):
   sdl_build = os.path.join(ports.get_build_dir(), 'sdl2')
   assert os.path.exists(sdl_build), 'You must use SDL2 to use SDL2_gfx'
   ports.fetch_project('sdl2_gfx', 'https://github.com/svn2github/sdl2_gfx/archive/' + TAG + '.zip', 'sdl2_gfx-' + TAG, sha512hash=HASH)
-  libname = ports.get_lib_name('libSDL2_gfx')
 
-  def create():
+  def create(final):
     logging.info('building port: sdl2_gfx')
 
     source_path = os.path.join(ports.get_dir(), 'sdl2_gfx', 'sdl2_gfx-' + TAG)
-    dest_path = os.path.join(shared.Cache.get_path('ports-builds'), 'sdl2_gfx')
+    dest_path = os.path.join(ports.get_build_dir(), 'sdl2_gfx')
 
     shutil.rmtree(dest_path, ignore_errors=True)
     shutil.copytree(source_path, dest_path)
-    final = os.path.join(dest_path, libname)
     ports.build_port(dest_path, final, [dest_path], exclude_dirs=['test'])
 
     ports.install_headers(source_path, target='SDL2')
-    return final
 
-  return [shared.Cache.get(libname, create)]
+  return [shared.Cache.get_lib('libSDL2_gfx.a', create)]
 
 
 def clear(ports, settings, shared):
-  shared.Cache.erase_file(ports.get_lib_name('libSDL2_gfx'))
+  shared.Cache.erase_lib('libSDL2_gfx.a')
 
 
 def process_args(ports):
