@@ -8,13 +8,11 @@ import hashlib
 import itertools
 import logging
 import os
-import re
 import shutil
 import sys
-import tarfile
-import zipfile
 from glob import iglob
 
+from .toolchain_profiler import ToolchainProfiler
 from . import shared, building, ports, config, utils
 from . import deps_info, tempfiles
 from . import diagnostics
@@ -22,6 +20,7 @@ from tools.shared import mangle_c_symbol_name, demangle_c_symbol_name
 
 logger = logging.getLogger('system_libs')
 
+# Files that are part of libsockets.a and so should be excluded from libc.a
 LIBC_SOCKETS = ['socket.c', 'socketpair.c', 'shutdown.c', 'bind.c', 'connect.c',
                 'listen.c', 'accept.c', 'getsockname.c', 'getpeername.c', 'send.c',
                 'recv.c', 'sendto.c', 'recvfrom.c', 'sendmsg.c', 'recvmsg.c',
@@ -54,8 +53,10 @@ def dir_is_newer(dir_a, dir_b):
   return newest_a < newest_b
 
 
-def get_cflags(force_object_files=False):
-  flags = []
+def get_base_cflags(force_object_files=False):
+  # Always build system libraries with debug information.  Non-debug builds
+  # will ignore this at link time because we link with `-strip-debug`.
+  flags = ['-g']
   if shared.Settings.LTO and not force_object_files:
     flags += ['-flto=' + shared.Settings.LTO]
   if shared.Settings.RELOCATABLE:
@@ -65,19 +66,16 @@ def get_cflags(force_object_files=False):
   return flags
 
 
-def run_one_command(cmd):
-  # Helper function used by run_build_commands.
-  if shared.EM_BUILD_VERBOSE:
-    print(shared.shlex_join(cmd))
+def clean_env():
   # building system libraries and ports should be hermetic in that it is not
-  # affected by things like EMMAKEN_CFLAGS which the user may have set
+  # affected by things like EMMAKEN_CFLAGS which the user may have set.
+  # At least one port also uses autoconf (harfbuzz) so we also need to clear
+  # CFLAGS/LDFLAGS which we don't want to effect the inner call to configure.
   safe_env = os.environ.copy()
-  for opt in ['EMMAKEN_CFLAGS', 'EMMAKEN_JUST_CONFIGURE']:
+  for opt in ['CFLAGS', 'LDFLAGS', 'EMCC_CFLAGS', 'EMMAKEN_CFLAGS', 'EMMAKEN_JUST_CONFIGURE']:
     if opt in safe_env:
       del safe_env[opt]
-  # TODO(sbc): Remove this one we remove the test_em_config_env_var test
-  cmd.append('-Wno-deprecated')
-  shared.run_process(cmd, env=safe_env)
+  return safe_env
 
 
 def run_build_commands(commands):
@@ -85,17 +83,7 @@ def run_build_commands(commands):
   # headers are installed.  This prevents each sub-process from attempting
   # to setup the sysroot itself.
   ensure_sysroot()
-  cores = min(len(commands), building.get_num_cores())
-  if cores <= 1 or shared.DEBUG:
-    for command in commands:
-      run_one_command(command)
-  else:
-    pool = building.get_multiprocessing_pool()
-    # https://stackoverflow.com/questions/1408356/keyboard-interrupts-with-pythons-multiprocessing-pool
-    # https://bugs.python.org/issue8296
-    # 999999 seconds (about 11 days) is reasonably huge to not trigger actual timeout
-    # and is smaller than the maximum timeout value 4294967.0 for Python 3 on Windows (threading.TIMEOUT_MAX)
-    pool.map_async(run_one_command, commands, chunksize=1).get(999999)
+  shared.run_multiple_processes(commands, env=clean_env())
 
 
 def create_lib(libname, inputs):
@@ -113,30 +101,19 @@ def create_lib(libname, inputs):
     raise Exception('unknown suffix ' + libname)
 
 
-def read_symbols(path):
-  with open(path) as f:
-    content = f.read()
-
-    # Require that Windows newlines should not be present in a symbols file, if running on Linux or macOS
-    # This kind of mismatch can occur if one copies a zip file of Emscripten cloned on Windows over to
-    # a Linux or macOS system. It will result in Emscripten linker getting confused on stray \r characters,
-    # and be unable to link any library symbols properly. We could harden against this by .strip()ping the
-    # opened files, but it is possible that the mismatching line endings can cause random problems elsewhere
-    # in the toolchain, hence abort execution if so.
-    if os.name != 'nt' and '\r\n' in content:
-      raise Exception('Windows newlines \\r\\n detected in symbols file "' + path + '"! This could happen for example when copying Emscripten checkout from Windows to Linux or macOS. Please use Unix line endings on checkouts of Emscripten on Linux and macOS!')
-
-    return building.parse_symbols(content).defs
-
-
 def get_wasm_libc_rt_files():
-  # Static linking is tricky with LLVM, since e.g. memset might not be used
-  # from libc, but be used as an intrinsic, and codegen will generate a libc
-  # call from that intrinsic *after* static linking would have thought it is
-  # all in there. In asm.js this is not an issue as we do JS linking anyhow,
-  # and have asm.js-optimized versions of all the LLVM intrinsics. But for
-  # wasm, we need a better solution. For now, make another archive that gets
-  # included at the same time as compiler-rt.
+  # Combining static linking with LTO is tricky under LLVM.  The codegen that
+  # happens during LTO can generate references to new symbols that didn't exist
+  # in the linker inputs themselves.
+  # These symbols are called libcalls in LLVM and are the result of intrinsics
+  # and builtins at the LLVM level.  These libcalls cannot themselves be part
+  # of LTO because once the linker is running the LTO phase new bitcode objects
+  # cannot be added to link.  Another way of putting it: by the time LTO happens
+  # the decision about which bitcode symbols to compile has already been made.
+  # See: https://bugs.llvm.org/show_bug.cgi?id=44353.
+  # To solve this we put all such libcalls in a separate library that, like
+  # compiler-rt, is never compiled as LTO/bitcode (see force_object_files in
+  # CompilerRTLibrary).
   # Note that this also includes things that may be depended on by those
   # functions - fmin uses signbit, for example, so signbit must be here (so if
   # fmin is added by codegen, it will have all it needs).
@@ -171,7 +148,7 @@ def get_wasm_libc_rt_files():
   return math_files + other_files + iprintf_files
 
 
-class Library(object):
+class Library:
   """
   `Library` is the base class of all system libraries.
 
@@ -240,10 +217,6 @@ class Library(object):
   # and left as None in an abstract library class, e.g. MTLibrary.
   name = None
 
-  # A set of symbols that this library exports. This will be set with a set
-  # returned by `read_symbols`.
-  symbols = set()
-
   # Set to true to prevent EMCC_FORCE_STDLIBS from linking this library.
   never_force = False
 
@@ -352,17 +325,21 @@ class Library(object):
     commands = []
     objects = []
     cflags = self.get_cflags()
+    base_flags = get_base_cflags()
     for src in self.get_files():
       o = os.path.join(build_dir, shared.unsuffixed_basename(src) + '.o')
       ext = shared.suffix(src)
-      if ext in ('.s', '.c'):
+      if ext in ('.s', '.S', '.c'):
         cmd = [shared.EMCC]
       else:
         cmd = [shared.EMXX]
-      if ext != '.s':
+      if ext in ('.s', '.S'):
+        cmd += base_flags
+        # TODO(sbc) There is an llvm bug that causes a crash when `-g` is used with
+        # assembly files that define wasm globals.
+        cmd.remove('-g')
+      else:
         cmd += cflags
-      elif shared.Settings.MEMORY64:
-        cmd += ['-s', 'MEMORY64=' + str(shared.Settings.MEMORY64)]
       commands.append(cmd + ['-c', src, '-o', o])
       objects.append(o)
     run_build_commands(commands)
@@ -394,7 +371,7 @@ class Library(object):
     Override and add any flags as needed to handle new variations.
     """
     cflags = self._inherit_list('cflags')
-    cflags += get_cflags(force_object_files=self.force_object_files)
+    cflags += get_base_cflags(force_object_files=self.force_object_files)
 
     if self.includes:
       cflags += ['-I' + shared.path_from_root(*path) for path in self._inherit_list('includes')]
@@ -531,7 +508,33 @@ class MTLibrary(Library):
     return super(MTLibrary, cls).get_default_variation(is_mt=shared.Settings.USE_PTHREADS, **kwargs)
 
 
-class exceptions(object):
+class OptimizedAggressivelyForSizeLibrary(Library):
+  def __init__(self, **kwargs):
+    self.is_optz = kwargs.pop('is_optz')
+    super(OptimizedAggressivelyForSizeLibrary, self).__init__(**kwargs)
+
+  def get_base_name(self):
+    name = super(OptimizedAggressivelyForSizeLibrary, self).get_base_name()
+    if self.is_optz:
+      name += '-optz'
+    return name
+
+  def get_cflags(self):
+    cflags = super(OptimizedAggressivelyForSizeLibrary, self).get_cflags()
+    if self.is_optz:
+      cflags += ['-DEMSCRIPTEN_OPTIMIZE_FOR_OZ']
+    return cflags
+
+  @classmethod
+  def vary_on(cls):
+    return super(OptimizedAggressivelyForSizeLibrary, cls).vary_on() + ['is_optz']
+
+  @classmethod
+  def get_default_variation(cls, **kwargs):
+    return super(OptimizedAggressivelyForSizeLibrary, cls).get_default_variation(is_optz=shared.Settings.SHRINK_LEVEL >= 2, **kwargs)
+
+
+class exceptions:
   """
   This represents exception handling mode of Emscripten. Currently there are
   three modes of exception handling:
@@ -577,9 +580,9 @@ class NoExceptLibrary(Library):
   @classmethod
   def variations(cls, **kwargs):
     combos = super(NoExceptLibrary, cls).variations()
-    # TODO Reenable 'wasm' option after the new EH implementation is stabilized
     return ([dict(eh_mode=exceptions.none, **combo) for combo in combos] +
-            [dict(eh_mode=exceptions.emscripten, **combo) for combo in combos])
+            [dict(eh_mode=exceptions.emscripten, **combo) for combo in combos] +
+            [dict(eh_mode=exceptions.wasm, **combo) for combo in combos])
 
   @classmethod
   def get_default_variation(cls, **kwargs):
@@ -669,13 +672,13 @@ class libc(AsanInstrumentedLibrary, MuslInternalLibrary, MTLibrary):
     ignore = [
         'ipc', 'passwd', 'thread', 'signal', 'sched', 'ipc', 'time', 'linux',
         'aio', 'exit', 'legacy', 'mq', 'search', 'setjmp', 'env',
-        'ldso', 'conf'
+        'ldso'
     ]
 
     # individual files
     ignore += [
         'memcpy.c', 'memset.c', 'memmove.c', 'getaddrinfo.c', 'getnameinfo.c',
-        'inet_addr.c', 'res_query.c', 'res_querydomain.c', 'gai_strerror.c',
+        'res_query.c', 'res_querydomain.c', 'gai_strerror.c',
         'proto.c', 'gethostbyaddr.c', 'gethostbyaddr_r.c', 'gethostbyname.c',
         'gethostbyname2_r.c', 'gethostbyname_r.c', 'gethostbyname2.c',
         'alarm.c', 'syscall.c', '_exit.c', 'popen.c',
@@ -708,18 +711,14 @@ class libc(AsanInstrumentedLibrary, MuslInternalLibrary, MTLibrary):
     ignore = set(ignore)
     # TODO: consider using more math code from musl, doing so makes box2d faster
     for dirpath, dirnames, filenames in os.walk(musl_srcdir):
+      # Don't recurse into ingored directories
+      remove = [d for d in dirnames if d in ignore]
+      for r in remove:
+        dirnames.remove(r)
+
       for f in filenames:
-        if f.endswith('.c'):
-          if f in ignore:
-            continue
-          dir_parts = os.path.split(dirpath)
-          cancel = False
-          for part in dir_parts:
-            if part in ignore:
-              cancel = True
-              break
-          if not cancel:
-            libc_files.append(os.path.join(musl_srcdir, dirpath, f))
+        if f.endswith('.c') and f not in ignore:
+          libc_files.append(os.path.join(musl_srcdir, dirpath, f))
 
     # Allowed files from ignored modules
     libc_files += files_in_path(
@@ -739,7 +738,12 @@ class libc(AsanInstrumentedLibrary, MuslInternalLibrary, MTLibrary):
 
     libc_files += files_in_path(
         path_components=['system', 'lib', 'libc'],
-        filenames=['extras.c', 'wasi-helpers.c', 'emscripten_pthread.c'])
+        filenames=[
+          'extras.c',
+          'wasi-helpers.c',
+          'emscripten_pthread.c',
+          'emscripten_get_heap_size.c',
+        ])
 
     libc_files += files_in_path(
         path_components=['system', 'lib', 'pthread'],
@@ -805,7 +809,6 @@ class libc(AsanInstrumentedLibrary, MuslInternalLibrary, MTLibrary):
         path_components=['system', 'lib', 'pthread'],
         filenames=[
           'library_pthread.c',
-          'emscripten_tls_init.c',
           'emscripten_thread_state.s',
         ])
     else:
@@ -835,14 +838,13 @@ class libsockets(MuslInternalLibrary, MTLibrary):
     return [os.path.join(network_dir, x) for x in LIBC_SOCKETS]
 
 
-class libsockets_proxy(MuslInternalLibrary, MTLibrary):
+class libsockets_proxy(MTLibrary):
   name = 'libsockets_proxy'
 
   cflags = ['-Os']
 
   def get_files(self):
-    return [shared.path_from_root('system', 'lib', 'websocket', 'websocket_to_posix_socket.cpp'),
-            shared.path_from_root('system', 'lib', 'libc', 'musl', 'src', 'network', 'inet_addr.c')]
+    return [shared.path_from_root('system', 'lib', 'websocket', 'websocket_to_posix_socket.cpp')]
 
 
 class crt1(MuslInternalLibrary):
@@ -875,6 +877,18 @@ class crt1_reactor(MuslInternalLibrary):
     return super(crt1_reactor, self).can_use() and shared.Settings.STANDALONE_WASM
 
 
+class crtbegin(Library):
+  name = 'crtbegin'
+  cflags = ['-O2', '-s', 'USE_PTHREADS']
+  src_dir = ['system', 'lib', 'pthread']
+  src_files = ['emscripten_tls_init.c']
+
+  force_object_files = True
+
+  def get_ext(self):
+    return '.o'
+
+
 class libcxxabi(NoExceptLibrary, MTLibrary):
   name = 'libc++abi'
   cflags = [
@@ -1065,7 +1079,7 @@ class libgl(MTLibrary):
   name = 'libgl'
 
   src_dir = ['system', 'lib', 'gl']
-  src_files = ['gl.c', 'webgl1.c']
+  src_files = ['gl.c', 'webgl1.c', 'libprocaddr.c']
 
   cflags = ['-Oz']
 
@@ -1167,6 +1181,15 @@ class libfetch(MTLibrary):
     return [shared.path_from_root('system', 'lib', 'fetch', 'emscripten_fetch.cpp')]
 
 
+class libstb_image(Library):
+  name = 'libstb_image'
+  never_force = True
+  includes = [['third_party']]
+
+  def get_files(self):
+    return [shared.path_from_root('system', 'lib', 'stb_image.c')]
+
+
 class libasmfs(MTLibrary):
   name = 'libasmfs'
   never_force = True
@@ -1195,7 +1218,7 @@ class CompilerRTLibrary(Library):
   force_object_files = True
 
 
-class libc_rt_wasm(AsanInstrumentedLibrary, CompilerRTLibrary, MuslInternalLibrary):
+class libc_rt_wasm(OptimizedAggressivelyForSizeLibrary, AsanInstrumentedLibrary, CompilerRTLibrary, MuslInternalLibrary):
   name = 'libc_rt_wasm'
 
   def get_files(self):
@@ -1213,6 +1236,7 @@ class libubsan_minimal_rt_wasm(CompilerRTLibrary, MTLibrary):
 
 class libsanitizer_common_rt(CompilerRTLibrary, MTLibrary):
   name = 'libsanitizer_common_rt'
+  # TODO(sbc): We should not need musl-internal headers here.
   includes = [['system', 'lib', 'libc', 'musl', 'src', 'internal'],
               ['system', 'lib', 'compiler-rt', 'lib']]
   never_force = True
@@ -1338,10 +1362,7 @@ class libstandalonewasm(MuslInternalLibrary):
         filenames=['assert.c', 'atexit.c', 'exit.c']) + files_in_path(
         path_components=['system', 'lib', 'libc', 'musl', 'src', 'unistd'],
         filenames=['_exit.c'])
-    conf_files = files_in_path(
-        path_components=['system', 'lib', 'libc', 'musl', 'src', 'conf'],
-        filenames=['sysconf.c'])
-    return base_files + time_files + exit_files + conf_files
+    return base_files + time_files + exit_files
 
 
 class libjsmath(Library):
@@ -1399,7 +1420,7 @@ def handle_reverse_deps(input_files):
   warn_on_unexported_main(symbolses)
 
   if len(symbolses) == 0:
-    class Dummy(object):
+    class Dummy:
       defs = set()
       undefs = set()
     symbolses.append(Dummy())
@@ -1444,11 +1465,8 @@ def calculate(input_files, cxx, forced):
   if force_include:
     logger.debug('forcing stdlibs: ' + str(force_include))
 
-  for lib in force_include:
-    if lib not in system_libs_map:
-      shared.exit_with_error('invalid forced library: %s', lib)
-
-  def add_library(lib):
+  def add_library(libname):
+    lib = system_libs_map[libname]
     if lib.name in already_included:
       return
     already_included.add(lib.name)
@@ -1458,68 +1476,76 @@ def calculate(input_files, cxx, forced):
     need_whole_archive = lib.name in force_include and lib.get_ext() == '.a'
     libs_to_link.append((lib.get_path(), need_whole_archive))
 
+  if shared.Settings.USE_PTHREADS:
+    add_library('crtbegin')
+
+  if shared.Settings.SIDE_MODULE:
+    return [l[0] for l in libs_to_link]
+
   if shared.Settings.STANDALONE_WASM:
     if shared.Settings.EXPECT_MAIN:
-      add_library(system_libs_map['crt1'])
+      add_library('crt1')
     else:
-      add_library(system_libs_map['crt1_reactor'])
+      add_library('crt1_reactor')
 
   for forced in force_include:
-    add_library(system_libs_map[forced])
+    if forced not in system_libs_map:
+      shared.exit_with_error('invalid forced library: %s', forced)
+    add_library(forced)
 
   if only_forced:
-    add_library(system_libs_map['libc_rt_wasm'])
-    add_library(system_libs_map['libcompiler_rt'])
+    add_library('libc_rt_wasm')
+    add_library('libcompiler_rt')
   else:
     if shared.Settings.AUTO_NATIVE_LIBRARIES:
-      add_library(system_libs_map['libgl'])
-      add_library(system_libs_map['libal'])
-      add_library(system_libs_map['libhtml5'])
+      add_library('libgl')
+      add_library('libal')
+      add_library('libhtml5')
 
     sanitize = shared.Settings.USE_LSAN or shared.Settings.USE_ASAN or shared.Settings.UBSAN_RUNTIME
 
     # JS math must come before anything else, so that it overrides the normal
     # libc math.
     if shared.Settings.JS_MATH:
-      add_library(system_libs_map['libjsmath'])
+      add_library('libjsmath')
 
     # to override the normal libc printf, we must come before it
     if shared.Settings.PRINTF_LONG_DOUBLE:
-      add_library(system_libs_map['libprintf_long_double'])
+      add_library('libprintf_long_double')
 
-    add_library(system_libs_map['libc'])
-    add_library(system_libs_map['libcompiler_rt'])
+    add_library('libc')
+    add_library('libcompiler_rt')
     if cxx:
-      add_library(system_libs_map['libc++'])
+      add_library('libc++')
     if cxx or sanitize:
-      add_library(system_libs_map['libc++abi'])
+      add_library('libc++abi')
       if shared.Settings.EXCEPTION_HANDLING:
-        add_library(system_libs_map['libunwind'])
+        add_library('libunwind')
     if shared.Settings.MALLOC != 'none':
-      add_library(system_libs_map['libmalloc'])
+      add_library('libmalloc')
     if shared.Settings.STANDALONE_WASM:
-      add_library(system_libs_map['libstandalonewasm'])
-    add_library(system_libs_map['libc_rt_wasm'])
+      add_library('libstandalonewasm')
+    add_library('libc_rt_wasm')
 
     if shared.Settings.USE_LSAN:
       force_include.add('liblsan_rt')
-      add_library(system_libs_map['liblsan_rt'])
+      add_library('liblsan_rt')
 
     if shared.Settings.USE_ASAN:
       force_include.add('libasan_rt')
-      add_library(system_libs_map['libasan_rt'])
-      add_library(system_libs_map['libasan_js'])
+      add_library('libasan_rt')
+      add_library('libasan_js')
 
     if shared.Settings.UBSAN_RUNTIME == 1:
-      add_library(system_libs_map['libubsan_minimal_rt_wasm'])
+      add_library('libubsan_minimal_rt_wasm')
     elif shared.Settings.UBSAN_RUNTIME == 2:
-      add_library(system_libs_map['libubsan_rt'])
+      add_library('libubsan_rt')
 
     if shared.Settings.USE_LSAN or shared.Settings.USE_ASAN:
-      add_library(system_libs_map['liblsan_common_rt'])
+      add_library('liblsan_common_rt')
 
     if sanitize:
-      add_library(system_libs_map['libsanitizer_common_rt'])
+      add_library('libsanitizer_common_rt')
 
     # the sanitizer runtimes may call mmap, which will need a few things. sadly
     # the usual deps_info mechanism does not work since we scan only user files
@@ -1530,12 +1556,12 @@ def calculate(input_files, cxx, forced):
       shared.Settings.EXPORTED_FUNCTIONS.append(mangle_c_symbol_name('memset'))
 
     if shared.Settings.PROXY_POSIX_SOCKETS:
-      add_library(system_libs_map['libsockets_proxy'])
+      add_library('libsockets_proxy')
     else:
-      add_library(system_libs_map['libsockets'])
+      add_library('libsockets')
 
     if shared.Settings.USE_WEBGPU:
-      add_library(system_libs_map['libwebgpu_cpp'])
+      add_library('libwebgpu_cpp')
 
   # When LINKABLE is set the entire link command line is wrapped in --whole-archive by
   # building.link_ldd.  And since --whole-archive/--no-whole-archive processing does not nest we
@@ -1561,7 +1587,7 @@ def calculate(input_files, cxx, forced):
   return ret
 
 
-class Ports(object):
+class Ports:
   """emscripten-ports library management (https://github.com/emscripten-ports).
   """
 
@@ -1626,7 +1652,7 @@ class Ports(object):
       # this must only be called on a standard build command
       assert cmd[0] in (shared.EMCC, shared.EMXX)
       # add standard cflags, but also allow the cmd to override them
-      return cmd[:1] + get_cflags() + cmd[1:]
+      return cmd[:1] + get_base_cflags() + cmd[1:]
     run_build_commands([add_args(c) for c in commands])
 
   @staticmethod
@@ -1653,7 +1679,7 @@ class Ports(object):
   name_cache = set()
 
   @staticmethod
-  def fetch_project(name, url, subdir, is_tarbz2=False, sha512hash=None):
+  def fetch_project(name, url, subdir, sha512hash=None):
     # To compute the sha512 hash, run `curl URL | sha512sum`.
     fullname = os.path.join(Ports.get_dir(), name)
 
@@ -1695,7 +1721,7 @@ class Ports(object):
               Ports.clear_project_build(name)
             return
 
-    if is_tarbz2:
+    if url.endswith('.tar.bz2'):
       fullpath = fullname + '.tar.bz2'
     elif url.endswith('.tar.gz'):
       fullpath = fullname + '.tar.gz'
@@ -1724,27 +1750,27 @@ class Ports(object):
         if actual_hash != sha512hash:
           shared.exit_with_error('Unexpected hash: ' + actual_hash + '\n'
                                  'If you are updating the port, please update the hash in the port module.')
-      open(fullpath, 'wb').write(data)
+      with open(fullpath, 'wb') as f:
+        f.write(data)
 
-    def check_tag():
-      if is_tarbz2:
-        names = tarfile.open(fullpath, 'r:bz2').getnames()
-      elif url.endswith('.tar.gz'):
-        names = tarfile.open(fullpath, 'r:gz').getnames()
-      else:
-        names = zipfile.ZipFile(fullpath, 'r').namelist()
-
-      # check if first entry of the archive is prefixed with the same
-      # tag as we need so no longer download and recompile if so
-      return bool(re.match(subdir + r'(\\|/|$)', names[0]))
+    marker = os.path.join(fullname, '.emscripten_url')
 
     def unpack():
       logger.info('unpacking port: ' + name)
       shared.safe_ensure_dirs(fullname)
       shutil.unpack_archive(filename=fullpath, extract_dir=fullname)
+      with open(marker, 'w') as f:
+        f.write(url + '\n')
+
+    def up_to_date():
+      if os.path.exists(marker):
+        with open(marker) as f:
+          if f.read().strip() == url:
+            return True
+      return False
 
     # before acquiring the lock we have an early out if the port already exists
-    if os.path.exists(fullpath) and check_tag():
+    if up_to_date():
       return
 
     # main logic. do this under a cache lock, since we don't want multiple jobs to
@@ -1753,7 +1779,7 @@ class Ports(object):
       if os.path.exists(fullpath):
         # Another early out in case another process build the library while we were
         # waiting for the lock
-        if check_tag():
+        if up_to_date():
           return
         # file exists but tag is bad
         logger.warning('local copy of port is not correct, retrieving from remote server')
@@ -1773,24 +1799,6 @@ class Ports(object):
     shared.try_delete(os.path.join(Ports.get_build_dir(), name))
 
 
-# get all ports
-def get_ports(settings):
-  ret = []
-  needed = get_needed_ports(settings)
-
-  for port in dependency_order(needed):
-    if port.needed(settings):
-      try:
-        # ports return their output files, which will be linked, or a txt file
-        ret += [f for f in port.get(Ports, settings, shared) if not f.endswith('.txt')]
-      except Exception:
-        logger.error('a problem occurred when using an emscripten-ports library.  try to run `emcc --clear-ports` and then run this command again')
-        raise
-
-  ret.reverse()
-  return ret
-
-
 def dependency_order(port_list):
   # Perform topological sort of ports according to the dependency DAG
   port_map = {p.name: p for p in port_list}
@@ -1823,7 +1831,7 @@ def resolve_dependencies(port_set, settings):
         port_set.add(dep)
         add_deps(dep)
 
-  for port in list(port_set):
+  for port in port_set.copy():
     add_deps(port)
 
 
@@ -1842,7 +1850,34 @@ def build_port(port_name, settings):
     port.get(Ports, settings, shared)
 
 
-def add_ports_cflags(args, settings):
+def clear_port(port_name, settings):
+  port = ports.ports_by_name[port_name]
+  port.clear(Ports, settings, shared)
+
+
+def get_ports_libs(settings):
+  """Called add link time to calculate the list of port libraries.
+  Can have the side effect of building and installing the needed ports.
+  """
+  ret = []
+  needed = get_needed_ports(settings)
+
+  for port in dependency_order(needed):
+    if port.needed(settings):
+      # ports return their output files, which will be linked, or a txt file
+      ret += [f for f in port.get(Ports, settings, shared) if not f.endswith('.txt')]
+
+  ret.reverse()
+  return ret
+
+
+def add_ports_cflags(args, settings): # noqa: U100
+  """Called during compile phase add any compiler flags (e.g -Ifoo) needed
+  by the selected ports.  Can also add/change settings.
+
+  Can have the side effect of building and installing the needed ports.
+  """
+
   # Legacy SDL1 port is not actually a port at all but builtin
   if settings.USE_SDL == 1:
     args += ['-Xclang', '-iwithsysroot/include/SDL']
@@ -1855,8 +1890,6 @@ def add_ports_cflags(args, settings):
     port.get(Ports, settings, shared)
     args += port.process_args(Ports)
 
-  return args
-
 
 def show_ports():
   print('Available ports:')
@@ -1906,4 +1939,5 @@ def install_system_headers(stamp):
 
 
 def ensure_sysroot():
-  shared.Cache.get('sysroot_install.stamp', install_system_headers, what='system headers')
+  with ToolchainProfiler.profile_block('ensure_sysroot'):
+    shared.Cache.get('sysroot_install.stamp', install_system_headers, what='system headers')
