@@ -1,12 +1,21 @@
-import subprocess, os, time, sys, tempfile
-if sys.version_info.major == 2:
-  import response_file
-else:
-  from tools import response_file
+# Copyright 2016 The Emscripten Authors.  All rights reserved.
+# Emscripten is available under two separate licenses, the MIT license and the
+# University of Illinois/NCSA Open Source License.  Both these licenses can be
+# found in the LICENSE file.
 
-EM_PROFILE_TOOLCHAIN = int(os.getenv('EM_PROFILE_TOOLCHAIN')) if os.getenv('EM_PROFILE_TOOLCHAIN') != None else 0
+import subprocess
+import os
+import time
+import sys
+import tempfile
 
-if EM_PROFILE_TOOLCHAIN:
+sys.path.insert(1, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
+
+from tools import response_file
+
+EMPROFILE = int(os.getenv('EMPROFILE', '0'))
+
+if EMPROFILE:
   original_sys_exit = sys.exit
   original_subprocess_call = subprocess.call
   original_subprocess_check_call = subprocess.check_call
@@ -22,9 +31,9 @@ if EM_PROFILE_TOOLCHAIN:
     ToolchainProfiler.record_subprocess_spawn(pid, cmd)
     try:
       returncode = original_subprocess_call(cmd, *args, **kw)
-    except Exception as e:
+    except Exception:
       ToolchainProfiler.record_subprocess_finish(pid, 1)
-      raise e
+      raise
     ToolchainProfiler.record_subprocess_finish(pid, returncode)
     return returncode
 
@@ -35,7 +44,7 @@ if EM_PROFILE_TOOLCHAIN:
       ret = original_subprocess_check_call(cmd, *args, **kw)
     except Exception as e:
       ToolchainProfiler.record_subprocess_finish(pid, e.returncode)
-      raise e
+      raise
     ToolchainProfiler.record_subprocess_finish(pid, 0)
     return ret
 
@@ -46,40 +55,28 @@ if EM_PROFILE_TOOLCHAIN:
       ret = original_subprocess_check_output(cmd, *args, **kw)
     except Exception as e:
       ToolchainProfiler.record_subprocess_finish(pid, e.returncode)
-      raise e
+      raise
     ToolchainProfiler.record_subprocess_finish(pid, 0)
     return ret
 
-  class ProfiledPopen(object):
-    def __init__(self, args, bufsize=0, executable=None, stdin=None, stdout=None, stderr=None, preexec_fn=None, close_fds=False,
-                 shell=False, cwd=None, env=None, universal_newlines=False, startupinfo=None, creationflags=0):
-      self.process = original_Popen(args, bufsize, executable, stdin, stdout, stderr, preexec_fn, close_fds, shell, cwd, env, universal_newlines, startupinfo, creationflags)
-      self.pid = self.process.pid
+  class ProfiledPopen(original_Popen):
+    def __init__(self, args, *otherargs, **kwargs):
+      super(ProfiledPopen, self).__init__(args, *otherargs, **kwargs)
       ToolchainProfiler.record_subprocess_spawn(self.pid, args)
 
-    def communicate(self, input=None):
+    def communicate(self, *args, **kwargs):
       ToolchainProfiler.record_subprocess_wait(self.pid)
-      output = self.process.communicate(input)
-      self.returncode = self.process.returncode
+      output = super(ProfiledPopen, self).communicate(*args, **kwargs)
       ToolchainProfiler.record_subprocess_finish(self.pid, self.returncode)
       return output
 
-    def wait(self):
-      return self.process.wait()
-
-    def poll(self):
-      return self.process.poll()
-
-    def kill(self):
-      return self.process.kill()
-
-  exit = sys.exit = profiled_sys_exit
+  sys.exit = profiled_sys_exit
   subprocess.call = profiled_call
   subprocess.check_call = profiled_check_call
   subprocess.check_output = profiled_check_output
   subprocess.Popen = ProfiledPopen
 
-  class ToolchainProfiler(object):
+  class ToolchainProfiler:
     # Provide a running counter towards negative numbers for PIDs for which we don't know what the actual process ID is
     imaginary_pid_ = 0
     profiler_logs_path = None # Log file not opened yet
@@ -108,6 +105,14 @@ if EM_PROFILE_TOOLCHAIN:
       # children generate are virtually treated as if they were performed by the parent PID.
       return open(os.path.join(ToolchainProfiler.profiler_logs_path, 'toolchain_profiler.pid_' + str(os.getpid()) + '.json'), 'a')
 
+    @staticmethod
+    def escape_string(arg):
+      return arg.replace('\\', '\\\\').replace('"', '\\"')
+
+    @staticmethod
+    def escape_args(args):
+      return map(lambda arg: ToolchainProfiler.escape_string(arg), args)
+
     @staticmethod
     def record_process_start(write_log_entry=True):
       # For subprocessing.Pool.map() child processes, this points to the PID of the parent process that spawned
@@ -116,20 +121,22 @@ if EM_PROFILE_TOOLCHAIN:
       ToolchainProfiler.profiler_logs_path = os.path.join(tempfile.gettempdir(), 'emscripten_toolchain_profiler_logs')
       try:
         os.makedirs(ToolchainProfiler.profiler_logs_path)
-      except:
+      except OSError:
         pass
 
-      if ToolchainProfiler.process_start_recorded: return
+      if ToolchainProfiler.process_start_recorded:
+        return
       ToolchainProfiler.process_start_recorded = True
       ToolchainProfiler.block_stack = []
 
       if write_log_entry:
         with ToolchainProfiler.log_access() as f:
-          f.write('[\n{"pid":' + ToolchainProfiler.mypid_str + ',"subprocessPid":' + str(os.getpid()) + ',"op":"start","time":' + ToolchainProfiler.timestamp() + ',"cmdLine":["' + '","'.join(sys.argv).replace('\\', '\\\\') + '"]}')
+          f.write('[\n{"pid":' + ToolchainProfiler.mypid_str + ',"subprocessPid":' + str(os.getpid()) + ',"op":"start","time":' + ToolchainProfiler.timestamp() + ',"cmdLine":["' + '","'.join(ToolchainProfiler.escape_args(sys.argv)) + '"]}')
 
     @staticmethod
     def record_process_exit(returncode):
-      if ToolchainProfiler.process_exit_recorded: return
+      if ToolchainProfiler.process_exit_recorded:
+        return
       ToolchainProfiler.process_exit_recorded = True
 
       ToolchainProfiler.exit_all_blocks()
@@ -138,13 +145,15 @@ if EM_PROFILE_TOOLCHAIN:
 
     @staticmethod
     def record_subprocess_spawn(process_pid, process_cmdline):
-      response_cmdline = []
+      expanded_cmdline = []
       for item in process_cmdline:
         if item.startswith('@'):
-          response_cmdline += response_file.read_response_file(item)
+          expanded_cmdline += response_file.read_response_file(item)
+        else:
+          expanded_cmdline.append(item)
 
       with ToolchainProfiler.log_access() as f:
-        f.write(',\n{"pid":' + ToolchainProfiler.mypid_str + ',"subprocessPid":' + str(os.getpid()) + ',"op":"spawn","targetPid":' + str(process_pid) + ',"time":' + ToolchainProfiler.timestamp() + ',"cmdLine":["' + '","'.join(process_cmdline + response_cmdline).replace('\\', '\\\\') + '"]}')
+        f.write(',\n{"pid":' + ToolchainProfiler.mypid_str + ',"subprocessPid":' + str(os.getpid()) + ',"op":"spawn","targetPid":' + str(process_pid) + ',"time":' + ToolchainProfiler.timestamp() + ',"cmdLine":["' + '","'.join(ToolchainProfiler.escape_args(expanded_cmdline)) + '"]}')
 
     @staticmethod
     def record_subprocess_wait(process_pid):
@@ -165,7 +174,7 @@ if EM_PROFILE_TOOLCHAIN:
 
     @staticmethod
     def remove_last_occurrence_if_exists(lst, item):
-      for i in xrange(len(lst)):
+      for i in range(len(lst)):
         if lst[i] == item:
           lst.pop(i)
           return True
@@ -182,7 +191,7 @@ if EM_PROFILE_TOOLCHAIN:
       for b in ToolchainProfiler.block_stack[::-1]:
         ToolchainProfiler.exit_block(b)
 
-    class ProfileBlock(object):
+    class ProfileBlock:
       def __init__(self, block_name):
         self.block_name = block_name
 
@@ -194,7 +203,7 @@ if EM_PROFILE_TOOLCHAIN:
 
     @staticmethod
     def profile_block(block_name):
-      return ToolchainProfiler.ProfileBlock(block_name)
+      return ToolchainProfiler.ProfileBlock(ToolchainProfiler.escape_string(block_name))
 
     @staticmethod
     def imaginary_pid():
@@ -202,9 +211,7 @@ if EM_PROFILE_TOOLCHAIN:
       return ToolchainProfiler.imaginary_pid_
 
 else:
-  exit = sys.exit
-
-  class ToolchainProfiler(object):
+  class ToolchainProfiler:
     @staticmethod
     def record_process_start():
       pass
@@ -233,11 +240,13 @@ else:
     def exit_block(block_name):
       pass
 
-    class ProfileBlock(object):
+    class ProfileBlock:
       def __init__(self, block_name):
         pass
+
       def __enter__(self):
         pass
+
       def __exit__(self, type, value, traceback):
         pass
 
